@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -17,9 +18,8 @@ from __future__ import annotations
 
 import logging
 import random
-from copy import deepcopy
 from dataclasses import dataclass
-from typing import Any, List, Optional, Tuple, Iterable
+from typing import Any, List, Optional, Tuple
 
 import torch
 from flsim.channels.base_channel import IdentityChannel
@@ -38,20 +38,19 @@ from flsim.optimizers.local_optimizers import (
     LocalOptimizerSGDConfig,
 )
 from flsim.optimizers.optimizer_scheduler import (
+    ConstantLRSchedulerConfig,
     OptimizerScheduler,
     OptimizerSchedulerConfig,
-    ConstantLRSchedulerConfig,
 )
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
-from flsim.utils.cuda import ICudaStateManager, DEFAULT_CUDA_MANAGER
+from flsim.utils.config_utils import fullclassname, init_self_cfg
+from flsim.utils.cuda import DEFAULT_CUDA_MANAGER, ICudaStateManager
 from flsim.utils.fl.common import FLModelParamUtils
 from hydra.utils import instantiate
 from omegaconf import OmegaConf
 
 
 class Client:
-    logger: logging.Logger = Logger.get_logger(__name__)
+    logger = Logger.get_logger(__name__)
 
     def __init__(
         self,
@@ -78,10 +77,10 @@ class Client:
             **OmegaConf.structured(NeverTimeOutSimulatorConfig())
         )
         self.store_last_updated_model = store_last_updated_model
-        self.name = name or "unnamed_client"
+        self._name = name or "unnamed_client"
 
         # base lr needs to match LR in optimizer config, overwrite it
-        # pyre-fixme[16]: `Client` has no attribute `cfg`.
+        # pyre-ignore [16]
         self.cfg.lr_scheduler.base_lr = self.cfg.optimizer.lr
         self.per_example_training_time = (
             self.timeout_simulator.simulate_per_example_training_time()
@@ -89,8 +88,11 @@ class Client:
         self.ref_model = None
         self.num_samples = 0
         self.times_selected = 0
+        # Tracks client state history
+        # Key: i-th time the client is selected for training
+        # Value: client state (i.e. model delta, client weight, optimizer used)
         self._tracked = {}
-        self._last_updated_model = None
+        self.last_updated_model = None
         self.logger.setLevel(logging.INFO)
 
     @classmethod
@@ -108,19 +110,9 @@ class Client:
 
     @property
     def model_deltas(self) -> List[IFLModel]:
-        """
-        return the stored deltas for all rounds that
-        this user was selected.
-        """
+        """Return the stored deltas for all rounds that this user was selected."""
         return [self._tracked[s]["delta"] for s in range(self.times_selected)]
 
-    @property
-    def last_updated_model(self):
-        """
-        return the most recent model on the client
-        """
-        return self._last_updated_model
-
     @property
     def optimizers(self) -> List[Any]:
         """Look at {self.model}"""
@@ -131,44 +123,83 @@ class Client:
         """Look at {self.model}"""
         return [self._tracked[s]["weight"] for s in range(self.times_selected)]
 
-    def generate_local_update(
-        self, model: IFLModel, metric_reporter: Optional[IFLMetricsReporter] = None
-    ) -> Tuple[IFLModel, float]:
-        r"""
-        wrapper around all functions called on a client for generating an
-        updated local model.
+    @property
+    def name(self) -> str:
+        return self._name
 
-        Note:
-        -----
-        Only pass a ``metric_reporter`` if reporting is needed, i.e.
-        report_metrics will be called ont the reporter, o.w. reports will be
-        accumulated in memory.
+    def generate_local_update(
+        self, message: Message, metrics_reporter: Optional[IFLMetricsReporter] = None
+    ) -> Tuple[IFLModel, float]:
+        """Wrapper around all functions called on a client for generating an updated
+        local model.
+
+        Args:
+            message: Message object. Must include the global model with optional metadata.
+
+        NOTE:
+            Only pass a `metrics_reporter` if reporting is needed, i.e. report_metrics
+            will be called on the reporter; else reports will be accumulated in memory.
         """
-        # 1. pass through channel, set initial state
-        updated_model = self.receive_through_channel(model)
-        # 2. set up model and optimizer in the client
-        updated_model, optim, optim_scheduler = self.prepare_for_training(updated_model)
-        # 3. kick off training on client
-        updated_model, weight = self.train(
-            updated_model, optim, optim_scheduler, metric_reporter
+        model = message.model
+
+        updated_model, weight, optimizer = self.copy_and_train_model(
+            model, metrics_reporter=metrics_reporter
         )
         # 4. Store updated model if being tracked
         if self.store_last_updated_model:
-            self._last_updated_model = deepcopy(updated_model)
-        # 5. compute delta
+            self.last_updated_model = FLModelParamUtils.clone(updated_model)
+
+        # 5. Compute model delta
         delta = self.compute_delta(
             before=model, after=updated_model, model_to_save=updated_model
         )
-        # 6. track state of the client
-        self.track(delta=delta, weight=weight, optimizer=optim)
+
+        # 6. Track the state of the client
+        self.track(delta=delta, weight=weight, optimizer=optimizer)
+
         return delta, weight
 
+    def copy_and_train_model(
+        self,
+        model: IFLModel,
+        epochs: Optional[int] = None,
+        optimizer: Optional[torch.optim.Optimizer] = None,
+        optimizer_scheduler: Optional[OptimizerScheduler] = None,
+        metrics_reporter: Optional[IFLMetricsReporter] = None,
+    ) -> Tuple[IFLModel, float, torch.optim.Optimizer]:
+        """Copy the source model to client-side model and use it to train on the
+        client's train split.
+
+        NOTE: Optional optimizer and optimizer_scheduler are there for easier testing
+
+        Returns:
+            (trained model, client's weight, optimizer used)
+        """
+        # 1. Pass source model through channel and use it to set client-side model state
+        updated_model = self.receive_through_channel(model)
+        # 2. Set up model and default optimizer in the client
+        updated_model, default_optim, default_scheduler = self.prepare_for_training(
+            updated_model
+        )
+        optim = default_optim if optimizer is None else optimizer
+        optim_scheduler = (
+            default_scheduler if optimizer_scheduler is None else optimizer_scheduler
+        )
+
+        # 3. Kick off training on client
+        updated_model, weight = self.train(
+            updated_model,
+            optim,
+            optim_scheduler,
+            metrics_reporter=metrics_reporter,
+            epochs=epochs,
+        )
+        return updated_model, weight, optim
+
     def compute_delta(
         self, before: IFLModel, after: IFLModel, model_to_save: IFLModel
     ) -> IFLModel:
-        """
-        Computes the delta between the before training and after training model
-        """
+        """Computes the delta of the model before and after training."""
         FLModelParamUtils.subtract_model(
             minuend=before.fl_get_module(),
             subtrahend=after.fl_get_module(),
@@ -177,33 +208,33 @@ class Client:
         return model_to_save
 
     def receive_through_channel(self, model: IFLModel) -> IFLModel:
+        """Receives a reference to a state (referred to as model state_dict) over the
+        channel. Any channel effect is applied as part of this function.
         """
-        Receives a reference to a state (refered to a model state_dict)
-        over the channel. Any channel effect is applied as part of this
-        receive function.
-        """
-        # keep a reference to global model
+        # Keep a reference to global model
         self.ref_model = model
 
-        # need to deepcopy the model because it's a reference to the global model
-        # modifying model will moidify the global model
-        message = self.channel.server_to_client(Message(model=deepcopy(model)))
+        # Need to clone the model because it's a reference to the global model; else
+        # modifying model will modify the global model.
+        message = self.channel.server_to_client(
+            Message(model=FLModelParamUtils.clone(model))
+        )
 
         return message.model
 
     def prepare_for_training(
         self, model: IFLModel
     ) -> Tuple[IFLModel, torch.optim.Optimizer, OptimizerScheduler]:
+        """Prepare for training by:
+        1. Instantiating a model on a correct compute device
+        2. Creating an optimizer
         """
-        1- instansiate a model with the given initial state
-        2- create an optimizer
-        """
-        # inform cuda_state_manager that we're about to train a model
-        # it may move model to GPU
+        # Inform cuda_state_manager that we're about to train a model; it may involve
+        # moving the model to GPU.
         self.cuda_state_manager.before_train_or_eval(model)
-        # put model in train mode
+        # Put model in train mode
         model.fl_get_module().train()
-        # create optimizer
+        # Create optimizer
         # pyre-fixme[16]: `Client` has no attribute `cfg`.
         optimizer = instantiate(self.cfg.optimizer, model=model.fl_get_module())
         optimizer_scheduler = instantiate(self.cfg.lr_scheduler, optimizer=optimizer)
@@ -211,7 +242,7 @@ class Client:
 
     def get_total_training_time(self) -> float:
         return self.timeout_simulator.simulate_training_time(
-            self.per_example_training_time, self.dataset.num_examples()
+            self.per_example_training_time, self.dataset.num_train_examples()
         )
 
     def stop_training(self, num_examples_processed) -> bool:
@@ -225,66 +256,92 @@ class Client:
         model: IFLModel,
         optimizer: Any,
         optimizer_scheduler: OptimizerScheduler,
-        metric_reporter: Optional[IFLMetricsReporter] = None,
+        metrics_reporter: Optional[IFLMetricsReporter] = None,
+        epochs: Optional[int] = None,
     ) -> Tuple[IFLModel, float]:
+        """Trains the client-side model for a number of epochs.
+        Returns:
+            (trained model, weight for this client)
+            Currently, weight for this client is the number of samples it trained on.
+            This might be a bad strategy because it favors users with more data, and
+            there might be privacy implications.
+        """
+        # Total number of training samples
         total_samples = 0
-        # NOTE currently weight = total_sampls, this might be a bad strategy
-        # plus there are privcay implications that must be taken into account.
-        num_examples_processed = 0  # number of examples processed during training
-        # pyre-fixme[16]: `Client` has no attribute `cfg`.
-        for epoch in range(self.cfg.epochs):
+
+        # Total number of sampled processed during training
+        # In general, this equals total_samples * epochs
+        num_examples_processed = 0
+
+        # pyre-ignore[16]: `Client` has no attribute `cfg`.
+        epochs = epochs if epochs is not None else self.cfg.epochs
+        if self.seed is not None:
+            torch.manual_seed(self.seed)
+
+        assert self.dataset.num_train_batches() > 0, "Client has no training data"
+
+        for epoch in range(epochs):
             if self.stop_training(num_examples_processed):
                 break
 
-            # if user has too many examples and times-out, we want to process
-            # different portion of the dataset each time
-            dataset = list(iter(self.dataset))
+            dataset = list(self.dataset.train_data())
+            # If the user has too many examples and times-out, we want to process a
+            # different portion of the dataset each time, hence shuffle batch order.
             if self.cfg.shuffle_batch_order:
                 random.shuffle(dataset)
             for batch in dataset:
-                # TODO use an independent random generator
-                if self.seed is not None:
-                    torch.manual_seed(self.seed)
-
                 sample_count = self._batch_train(
                     model=model,
                     optimizer=optimizer,
                     training_batch=batch,
                     epoch=epoch,
-                    metric_reporter=metric_reporter,
+                    metrics_reporter=metrics_reporter,
                     optimizer_scheduler=optimizer_scheduler,
                 )
+                # Optional post-processing after training on a batch
                 self.post_batch_train(epoch, model, sample_count, optimizer)
+                # Only add in epoch 0 because we want to get the training set size
                 total_samples += 0 if epoch else sample_count
                 num_examples_processed += sample_count
-                # stop training depending on time-out condition
+                # Stop training depending on time-out condition
                 if self.stop_training(num_examples_processed):
                     break
-        # tell cuda manager we're done with training
+
+        # Tell cuda manager we're done with training
         # cuda manager may move model out of GPU memory if needed
         self.cuda_state_manager.after_train_or_eval(model)
         self.logger.debug(
-            f"Processed {num_examples_processed} of {self.dataset.num_examples()}"
+            f"Processed {num_examples_processed} of {self.dataset.num_train_examples()}"
         )
+
+        # Optional post-processing after the entire training is done
         self.post_train(model, total_samples, optimizer)
-        # if training stops early, used partial training weight
+
+        # If training stops early (i.e. `num_examples_processed < total_samples`), use
+        # `num_examples_processed` instead as weight.
         example_weight = min([num_examples_processed, total_samples])
+
         return model, float(example_weight)
 
     def post_train(self, model: IFLModel, total_samples: int, optimizer: Any):
+        """Post-processing after the entire training on multiple epochs is done."""
         pass
 
     def post_batch_train(
         self, epoch: int, model: IFLModel, sample_count: int, optimizer: Any
     ):
+        """Post-processing after training on one batch is done."""
         pass
 
     def track(self, delta: IFLModel, weight: float, optimizer: Any):
-        """Tracks metric when the client is selected multiple times"""
+        """Tracks the client state each time the client is selected.
+        Modifies `self_tracked`, where keys are the i-th time this client is selected
+        and values are the state of the client.
+        """
         # pyre-fixme[16]: `Client` has no attribute `cfg`.
         if self.cfg.store_models_and_optimizers:
             self._tracked[self.times_selected] = {
-                "delta": deepcopy(delta),
+                "delta": FLModelParamUtils.clone(delta),
                 "weight": weight,
                 "optimizer": optimizer,
             }
@@ -292,20 +349,11 @@ class Client:
 
     def eval(
         self,
-        model: Optional[IFLModel] = None,
-        dataset: Optional[Iterable] = None,
-        metric_reporter: Optional[IFLMetricsReporter] = None,
+        model: IFLModel,
+        dataset: Optional[IFLUserData] = None,
+        metrics_reporter: Optional[IFLMetricsReporter] = None,
     ):
-        """
-        Evaluate the given `model` with the given `dataset`. `model` defaults
-        to the current global_model if nothing is provided, and `dataset`
-        defaults to client's dataset.
-        """
-        # Note here we play a trick, model and dataset provided are not
-        # passed through a channel, in ptactice this needs to be done if
-        # either the model or dataset is not what is currently being held
-        # locally at the client.
-        model = model or self.ref_model
+        """Evaluates client-side model with its evaluation data split."""
         data = dataset or self.dataset
         self.cuda_state_manager.before_train_or_eval(model)
         with torch.no_grad():
@@ -313,11 +361,10 @@ class Client:
                 torch.manual_seed(self.seed)
 
             model.fl_get_module().eval()
-            for batch in data:
+            for batch in data.eval_data():
                 batch_metrics = model.get_eval_metrics(batch)
-                if metric_reporter is not None:
-                    # TODO MM make sure metric reporter is multi-process safe.
-                    metric_reporter.add_batch_metrics(batch_metrics)
+                if metrics_reporter is not None:
+                    metrics_reporter.add_batch_metrics(batch_metrics)
         model.fl_get_module().train()
         self.cuda_state_manager.after_train_or_eval(model)
 
@@ -327,31 +374,22 @@ class Client:
         optimizer,
         training_batch,
         epoch,
-        metric_reporter,
+        metrics_reporter,
         optimizer_scheduler,
     ) -> int:
-        """Trainer for NewDocModel based FL Tasks
-        Run a single iteration of minibatch-gradient descent on a single user.
-        Compatible with the new tasks in which model is reponsible to
-        arrange its inputs, targets and context.
-        Return number of examples in the batch.
+        """Trains the client-side model for one batch.
+        Compatible with the new tasks in which the model is responsible for
+        arranging its inputs, targets and context.
+        Returns:
+            Number of samples in the batch.
         """
-        if self.logger.isEnabledFor(logging.DEBUG):
-            self.logger.debug(
-                "L1-norm of Parameters of initial state:",
-                sum(p.abs().sum() for p in self.ref_model.fl_get_module().parameters()),
-            )
-            self.logger.debug(
-                "L1-norm of Parameters before step:",
-                sum(p.abs().sum() for p in model.fl_get_module().parameters()),
-            )
-
         optimizer.zero_grad()
         batch_metrics = model.fl_forward(training_batch)
         loss = batch_metrics.loss
 
         loss.backward()
         # pyre-fixme[16]: `Client` has no attribute `cfg`.
+        # Optional gradient clipping
         if self.cfg.max_clip_norm_normalized is not None:
             max_norm = self.cfg.max_clip_norm_normalized
             FLModelParamUtils.clip_gradients(
@@ -359,38 +397,58 @@ class Client:
             )
 
         num_examples = batch_metrics.num_examples
-        # adjust lr and take a step
+        # Adjust lr and take a gradient step
         optimizer_scheduler.step(batch_metrics, model, training_batch, epoch)
         optimizer.step()
 
-        if metric_reporter is not None:
-            metric_reporter.add_batch_metrics(batch_metrics)
+        if metrics_reporter is not None:
+            metrics_reporter.add_batch_metrics(batch_metrics)
 
-        if self.logger.isEnabledFor(logging.DEBUG):
-            self.logger.debug(
-                "L1-norm of Parameters after step:",
-                sum(p.abs().sum() for p in model.fl_get_module().parameters()),
-            )
-            l2_norm_raw = FLModelParamUtils.get_gradient_l2_norm_raw(
-                model.fl_get_module()
-            )
-            l2_norm_normalized = FLModelParamUtils.get_gradient_l2_norm_normalized(
-                model.fl_get_module()
-            )
-            self.logger.debug(
-                f"Train Loss:{loss},"
-                f"GradNorm:{l2_norm_raw},"
-                f"NormalizedGradNorm:{l2_norm_normalized},",
-                f"NumExamples:{num_examples}",
-            )
         return num_examples
 
+    def full_dataset_gradient(self, module: IFLModel):
+        """Performs a pass over the entire training set and returns the average gradient.
+        Currently only called from SyncMimeServer.
+
+        Args:
+            module (IFLModel): Model over which to return gradients
+
+        Returns:
+            grads (nn.Module): Dummy module with the desired gradients contained in its
+                parameters (access via .grad).
+            num_examples (int): Number of training examples in the dataset.
+        """
+        # `grads` is a dummy module for storing the average gradient.
+        # Its network weights are never used.
+        module.fl_get_module().train()
+        grads = FLModelParamUtils.clone(module.fl_get_module())
+        grads.zero_grad()
+        num_examples = 0
+
+        for batch in self.dataset.train_data():
+            module.fl_get_module().zero_grad()
+            batch_metrics = module.fl_forward(batch)
+            batch_metrics.loss.backward()
+            FLModelParamUtils.multiply_gradient_by_weight(
+                module.fl_get_module(),
+                batch_metrics.num_examples,
+                module.fl_get_module(),
+            )
+            FLModelParamUtils.add_gradients(grads, module.fl_get_module(), grads)
+            num_examples += batch_metrics.num_examples
+        assert num_examples > 0, "Client has no training data"
+        # Average out the gradient over the client
+        FLModelParamUtils.multiply_gradient_by_weight(
+            model=grads, weight=1.0 / num_examples, model_to_save=grads
+        )
+        return grads, num_examples
+
 
 @dataclass
 class ClientConfig:
     _target_: str = fullclassname(Client)
     _recursive_: bool = False
-    epochs: int = 1  # No. of epochs for local training
+    epochs: int = 1  # Number of epochs for local training
     optimizer: LocalOptimizerConfig = LocalOptimizerConfig()
     lr_scheduler: OptimizerSchedulerConfig = OptimizerSchedulerConfig()
     max_clip_norm_normalized: Optional[float] = None  # gradient clip value
@@ -398,4 +456,4 @@ class ClientConfig:
     random_seed: Optional[int] = None  # random seed for deterministic response
     shuffle_batch_order: bool = False  # shuffle the ordering of batches
     store_models_and_optimizers: bool = False  # name clear
-    track_multiple_selection: bool = False  # track if client appears in 2+ rounds.
+    track_multiple_selection: bool = False  # track if the client appears in 2+ rounds.
