@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -14,14 +15,13 @@ from flsim.clients.base_client import ClientConfig
 from flsim.common.pytest_helper import (
     assertAlmostEqual,
     assertEqual,
-    assertTrue,
-    assertLessEqual,
     assertGreaterEqual,
+    assertLessEqual,
+    assertTrue,
 )
 from flsim.common.training_event_handler import TestAsyncTrainingEventHandler
 from flsim.common.training_simulator import AsyncTrainingSimulator
 from flsim.data.data_provider import FLDataProviderFromList, IFLDataProvider
-from flsim.tests.utils import check_inherit_logging_level
 from flsim.utils.async_trainer.async_user_selector import (
     AsyncUserSelector,
     RandomAsyncUserSelector,
@@ -38,6 +38,7 @@ from flsim.utils.async_trainer.training_event_generator import (
     PoissonAsyncTrainingStartTimeDistrConfig,
 )
 from flsim.utils.sample_model import MockFLModel
+from flsim.utils.test_utils import check_inherit_logging_level
 from flsim.utils.tests.helpers.test_training_simulator_utils import (  # noqa: B950 line is too long
     ConstantEventGenTestConfig,
     ConstantEventGenTestConfigPerUserGaussian,
@@ -47,10 +48,10 @@ from flsim.utils.tests.helpers.test_training_simulator_utils import (  # noqa: B
 from flsim.utils.timing.training_duration_distribution import (
     DurationDistributionConfig,
     PerExampleGaussianDurationDistributionConfig,
+    PerUserExponentialDurationDistributionConfig,
     PerUserGaussianDurationDistributionConfig,
     PerUserHalfNormalDurationDistributionConfig,
     PerUserUniformDurationDistributionConfig,
-    PerUserExponentialDurationDistributionConfig,
 )
 from omegaconf import OmegaConf
 
@@ -61,7 +62,7 @@ class Globals:
 
 
 @pytest.fixture(scope="class")
-def prepare_training_simulator_test_utils(request):
+def prepare_training_simulator_test_utils(request) -> None:
     torch.random.manual_seed(0)
     request.cls.shared_client_config = ClientConfig(
         epochs=1,
@@ -100,19 +101,21 @@ class TestTrainingSimulatorUtils:
         exp_start_time: int,
         exp_end_time: int,
         exp_training_state: TrainingState,
-    ):
+    ) -> None:
         assertEqual(
             # pyre-fixme[16]: `TrainingState` has no attribute `training_start_time`.
             training_event[Globals.DEVICE_STATE_IDX].training_schedule.start_time,
             exp_start_time,
         )
         assertEqual(
+            # pyre-fixme[16]: Item `TrainingState` of `Union[DeviceState,
+            #  TrainingState]` has no attribute `training_schedule`.
             training_event[Globals.DEVICE_STATE_IDX].training_schedule.end_time,
             exp_end_time,
         )
         assertEqual(training_event[Globals.TRAINING_STATE_IDX], exp_training_state)
 
-    def test_training_simulator(self):
+    def test_training_simulator(self) -> None:
         """Check that the priority queue in the training simulator works as
         expected by creating a training simulator with a known list of
         event start times and event durations.
@@ -150,6 +153,8 @@ class TestTrainingSimulatorUtils:
             job_scheduler=job_scheduler,
             user_selector=self.random_user_selector(data_provider=data_provider),
             event_generator=distr,
+            # pyre-fixme[16]: `TestTrainingSimulatorUtils` has no attribute
+            #  `shared_client_config`.
             shared_client_config=self.shared_client_config,
             num_train_end_events_per_epoch=len(event_list) - 1,
         )
@@ -220,7 +225,7 @@ class TestTrainingSimulatorUtils:
         assertAlmostEqual(job_scheduler.seqnum_diff_mean(), 2 / 3)
         assertAlmostEqual(job_scheduler.seqnum_std(), math.sqrt(24 / 27))
 
-    def test_async_stats(self):
+    def test_async_stats(self) -> None:
         """Check that the priority JobQueueStats functionality in the training
         simulator works as expected by creating a training simulator with a
         known list of event start times and event durations.
@@ -252,6 +257,8 @@ class TestTrainingSimulatorUtils:
             job_scheduler=job_scheduler,
             user_selector=self.random_user_selector(data_provider=data_provider),
             event_generator=distr,
+            # pyre-fixme[16]: `TestTrainingSimulatorUtils` has no attribute
+            #  `shared_client_config`.
             shared_client_config=self.shared_client_config,
             num_train_end_events_per_epoch=len(event_list) - 1,
         )
@@ -261,7 +268,7 @@ class TestTrainingSimulatorUtils:
 
     def test_sequential_training(self) -> None:
         """Check that in sequential training (where mean and SD of training
-        time is zero), jobs are truly trained sequentialy: i.e, if job A
+        time is zero), jobs are truly trained sequentially: i.e, if job A
         starts training, job A ends training before any other jobs start
         training
         """
@@ -525,19 +532,19 @@ class TestTrainingSimulatorUtils:
             [i[ds_idx].user_info.user_index for i in last_n_events], user_finishing_seq
         )
 
-    def test_training_duration_per_example_gaussian(self):
+    def test_training_duration_per_example_gaussian(self) -> None:
         pass
 
-    def test_training_duration_per_user_gaussian(self):
+    def test_training_duration_per_user_gaussian(self) -> None:
         pass
 
-    def test_training_end_training_start_relative_priority(self):
+    def test_training_end_training_start_relative_priority(self) -> None:
         r"""
         Test that when training end and training start events have the same time,
         training end event happens earlier (has lower priority)
         """
 
-    def test_logging_level(self):
+    def test_logging_level(self) -> None:
         num_users = 50
         training_start_time_distr = PoissonAsyncTrainingStartTimeDistrConfig(
             training_rate=10
@@ -561,6 +568,8 @@ class TestTrainingSimulatorUtils:
             job_scheduler=job_scheduler,
             user_selector=self.random_user_selector(data_provider=data_provider),
             event_generator=distr,
+            # pyre-fixme[16]: `TestTrainingSimulatorUtils` has no attribute
+            #  `shared_client_config`.
             shared_client_config=self.shared_client_config,
             num_train_end_events_per_epoch=num_users,
         )
@@ -570,9 +579,9 @@ class TestTrainingSimulatorUtils:
     def _training_staleness_distribution(
         self,
         duration_distr,
-        training_rate=1000,
-        num_users=10000,
-        examples_per_user=1,
+        training_rate: int = 1000,
+        num_users: int = 10000,
+        examples_per_user: int = 1,
     ):
         data_provider = self._create_data_provider(
             num_users=num_users, examples_per_user=examples_per_user
@@ -594,13 +603,15 @@ class TestTrainingSimulatorUtils:
             job_scheduler=handler,
             user_selector=RandomAsyncUserSelector(data_provider=data_provider),
             event_generator=distr,
+            # pyre-fixme[16]: `TestTrainingSimulatorUtils` has no attribute
+            #  `shared_client_config`.
             shared_client_config=self.shared_client_config,
             num_train_end_events_per_epoch=num_users,
         )
         training_sim.run_one_epoch()
         return np.array(handler.num_unseen_global_model_updates)
 
-    def test_per_user_half_normal_training_duration(self):
+    def test_per_user_half_normal_training_duration(self) -> None:
         """
         Test for staleness with half normal
 
@@ -609,7 +620,7 @@ class TestTrainingSimulatorUtils:
             heavy tailed and there is no concentration of users with low
             staleness more than usual
         Condition 2:
-            Check that the distribution has a long tail that approximate a guassian tail
+            Check that the distribution has a long tail that approximates a Gaussian tail
             All values should be within 6 std of the mean
         """
         num_users = 1000
@@ -632,7 +643,7 @@ class TestTrainingSimulatorUtils:
         # 2e-11 = e^(−k^2/2) = exp(-7^2/2)
         assertLessEqual(max(staleness), mean + 7 * sd)
 
-    def test_per_user_uniform_training_duration(self):
+    def test_per_user_uniform_training_duration(self) -> None:
         num_users = 2000
         training_rate = 10
         training_min = 0.0
@@ -654,7 +665,7 @@ class TestTrainingSimulatorUtils:
         assertLessEqual(max(staleness), mean + 4 * sd)
         assertGreaterEqual(min(staleness), mean - 4 * sd)
 
-    def test_per_user_exponential_training_duration(self):
+    def test_per_user_exponential_training_duration(self) -> None:
         for training_mean in [0.5, 1, 2]:
             num_users = 2000
             training_rate = 10
