@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -15,10 +16,8 @@ from flsim.common.logger import Logger
 from flsim.common.timeout_simulator import TimeOutSimulator
 from flsim.common.training_event_handler import AsyncTrainingEventHandler
 from flsim.utils.async_trainer.async_user_selector import AsyncUserSelector
-from flsim.utils.async_trainer.training_event_generator import (
-    IEventGenerator,
-)
-from flsim.utils.cuda import ICudaStateManager, DEFAULT_CUDA_MANAGER
+from flsim.utils.async_trainer.training_event_generator import IEventGenerator
+from flsim.utils.cuda import DEFAULT_CUDA_MANAGER, ICudaStateManager
 from tqdm import tqdm
 
 
@@ -68,7 +67,7 @@ class AsyncTrainingSimulator:
     AsyncJobScheduler’s on_training_start_function, put the DeviceState back
     to the min_heap, and log to the JobQueueStats.
     Otherwise, it simply ends the training. Note that this works, because
-    relevant_tmie is set to training end time after training is triggered.
+    relevant_time is set to training end time after training is triggered.
     """
 
     logger: logging.Logger = Logger.get_logger(__name__)
