@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -8,6 +9,7 @@ import flsim.configs  # noqa
 import omegaconf
 from flsim.common.pytest_helper import assertRaises
 from flsim.utils.sample_model import DummyAlphabetFLModel
+from hydra.errors import InstantiationException
 from hydra.experimental import compose, initialize
 from hydra.utils import instantiate
 
@@ -26,6 +28,6 @@ class TestConfigValidation:
                 ],
             )
         with assertRaises(
-            omegaconf.errors.MissingMandatoryValue,  # with Hydra 1.1
+            InstantiationException,  # with Hydra 1.1
         ):
             instantiate(cfg.trainer, model=DummyAlphabetFLModel(), cuda_enabled=False)
