@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -14,8 +15,7 @@ from typing import Any, Dict, Iterable, List, Tuple, TypeVar
 
 import numpy as np
 import torch
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
+from flsim.utils.config_utils import fullclassname, init_self_cfg
 from omegaconf import MISSING
 from torch.utils.data import Dataset
 
@@ -49,13 +49,14 @@ class FLDataSharder(abc.ABC):
         pass
 
     def shard_rows(self, data_rows: Shardable) -> Iterable[Tuple[str, Any]]:
-        """Partition a set of rows into mulitple sets using a sharding strategy.
+        """Partition a set of rows into multiple sets using a sharding strategy.
 
         Args:
             data_rows: Iterable[Dict[str, Any]]]: iterable over dictionary mapping column
             name to value.
         """
         shards = defaultdict(list)
+        # pyre-fixme[16]: `Dataset` has no attribute `__iter__`.
         for one_row in data_rows:
             for shard_id in self.shard_for_row(one_row):
                 shards[str(shard_id)].append(one_row)
