@@ -1,12 +1,13 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
 import abc
-from enum import Enum, auto
-from typing import Any, Dict, List, Optional, Union, Tuple
+from enum import auto, Enum
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 from flsim.interfaces.batch_metrics import IFLBatchMetrics
 from torch import Tensor
@@ -63,7 +64,7 @@ class Metric:
         The usage is simple, metrics need to be passed as named arguments
         to the function. The class will throw if the metrics are not
         any of the valid types: int, float, tensor of size 1, or a
-        dictionary of such types with string keys. This latter case is considerd
+        dictionary of such types with string keys. This latter case is considered
         a metric with sub metrics.
 
         Example:
@@ -112,6 +113,8 @@ class TrainingStage(Enum):
     TEST = auto()
     EVAL = auto()
     PER_CLIENT_EVAL = auto()
+    PERSONALIZED_EVAL = auto()  # personalized eval on eval users
+    PERSONALIZED_TEST = auto()  # personalized eval on test users
 
 
 class IFLMetricsReporter(abc.ABC):
@@ -125,7 +128,7 @@ class IFLMetricsReporter(abc.ABC):
     training aggregates all the data by calling aggregate() method, which gets
     a MetricsReporter of an user who just completed one’s own epoch. Then, after
     all users’ local epochs are completed, the global MetricsReporter completes
-    its global aggreation and report its metrics to given channels for that global
+    its global aggregation and report its metrics to given channels for that global
     epoch.
     Note: 1 global epoch consists of several rounds. In each round, we train
     a subset of users and each user goes through a number of local epochs, where
@@ -165,7 +168,7 @@ class IFLMetricsReporter(abc.ABC):
         """
         pass
 
-    # TODO: is this needed? Do we ever call this externally?
+    # TODO: Is this needed? Do we ever call this externally?
     @abc.abstractmethod
     def reset(self):
         """Clean up all aggregations so far."""
