@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -11,7 +12,10 @@ This interface is similar to torch.optim.Optimizer.
 
   Typical usage example:
 
-  optimizer = OptimizerType.create_optimizer(model=model, config=config)
+  optimizer =        self._optimizer = instantiate(
+            config=self.cfg.server_optimizer,
+            model=global_model.fl_get_module(),
+        )
   optimizer.step()
 """
 
@@ -24,7 +28,7 @@ from enum import Enum
 import torch
 import torch.nn as nn
 from flsim.optimizers.layerwise_optimizers import LAMB, LARS
-from flsim.utils.config_utils import fullclassname, init_self_cfg, is_target
+from flsim.utils.config_utils import fullclassname, init_self_cfg
 from omegaconf import MISSING
 
 
@@ -44,7 +48,7 @@ class IServerOptimizer(abc.ABC):
 
     @abc.abstractmethod
     @torch.no_grad()
-    def step(self, closure):
+    def step(self, closure, noise=None):
         r"""Performs a single optimization step (parameter update).
 
         Args:
@@ -63,7 +67,7 @@ class IServerOptimizer(abc.ABC):
 
         Args:
             set_to_none (bool): instead of setting to zero, set the grads to None.
-                This will in general have lower memory footprint, and can modestly improve performance.
+                This will in general have a lower memory footprint, and can modestly improve performance.
                 However, it changes certain behaviors. For example:
                 1. When the user tries to access a gradient and perform manual ops on it,
                 a None attribute or a Tensor full of 0s will behave differently.
@@ -76,7 +80,6 @@ class IServerOptimizer(abc.ABC):
         raise NotImplementedError
 
 
-# pyre-ignore[11] Annotation
 class FedAvgWithLROptimizer(IServerOptimizer, torch.optim.SGD):
     def __init__(self, *, model: nn.Module, **kwargs) -> None:
         init_self_cfg(
@@ -128,7 +131,6 @@ class FedAvgOptimizer(IServerOptimizer, torch.optim.SGD):
         return torch.optim.SGD.zero_grad(self, set_to_none)
 
 
-# pyre-ignore[11] Annotation
 class FedAdamOptimizer(IServerOptimizer, torch.optim.Adam):
     def __init__(self, *, model: nn.Module, **kwargs) -> None:
         init_self_cfg(
@@ -189,7 +191,7 @@ class FedLAMBOptimizer(IServerOptimizer, LAMB):
         init_self_cfg(
             self,
             component_class=__class__,
-            config_class=FedLARSOptimizerConfig,
+            config_class=FedLAMBOptimizerConfig,
             **kwargs,
         )
 
@@ -213,6 +215,105 @@ class FedLAMBOptimizer(IServerOptimizer, LAMB):
         return LAMB.zero_grad(self, set_to_none)
 
 
+class ServerFTRLOptimizer(IServerOptimizer, torch.optim.Optimizer):
+    """
+    :param params: parameter groups
+    :param momentum: if non-zero, use DP-FTRLM
+    :param record_last_noise: whether to record the last noise. for the tree completion trick.
+    """
+
+    def __init__(self, *, model: nn.Module, record_last_noise: bool, **kwargs) -> None:
+        init_self_cfg(
+            self,
+            component_class=__class__,
+            config_class=ServerFTRLOptimizerConfig,
+            **kwargs,
+        )
+
+        IServerOptimizer.__init__(self, model=model, **kwargs)
+        # pyre-ignore[28]
+        torch.optim.Optimizer.__init__(self, params=model.parameters(), defaults={})
+        # pyre-ignore[16]
+        self.momentum = self.cfg.momentum
+        self.lr = self.cfg.lr
+        self.record_last_noise = record_last_noise
+
+    def __setstate__(self, state):
+        super(ServerFTRLOptimizer, self).__setstate__(state)
+
+    def zero_grad(self, set_to_none: bool = False):
+        return torch.optim.Optimizer.zero_grad(self, set_to_none)
+
+    @torch.no_grad()
+    def step(self, noise, closure=None):
+        loss = None
+        if closure is not None:
+            with torch.enable_grad():
+                loss = closure()
+
+        for group in self.param_groups:
+            for p, nz in zip(group["params"], noise):
+                if p.grad is None:
+                    continue
+                d_p = p.grad
+
+                param_state = self.state[p]
+
+                if len(param_state) == 0:
+                    param_state["grad_sum"] = torch.zeros_like(
+                        d_p, memory_format=torch.preserve_format
+                    )
+                    param_state["model_sum"] = p.detach().clone(
+                        memory_format=torch.preserve_format
+                    )  # just record the initial model
+                    param_state["momentum"] = torch.zeros_like(
+                        p, memory_format=torch.preserve_format
+                    )
+                    if self.record_last_noise:
+                        param_state["last_noise"] = torch.zeros_like(
+                            p, memory_format=torch.preserve_format
+                        )  # record the last noise needed, in order for restarting
+
+                gs, ms = param_state["grad_sum"], param_state["model_sum"]
+                if self.momentum == 0:
+                    gs.add_(d_p)
+                    p.copy_(ms + (-gs - nz) / self.lr)
+                else:
+                    gs.add_(d_p)
+                    param_state["momentum"].mul_(self.momentum).add_(gs + nz)
+                    p.copy_(ms - param_state["momentum"] / self.lr)
+                if self.record_last_noise:
+                    param_state["last_noise"].copy_(nz)
+        return loss
+
+    @torch.no_grad()
+    def restart(self, last_noise=None):
+        """
+        Restart the tree.
+        :param last_noise: the last noise to be added. If none, use the last noise recorded.
+        """
+        assert last_noise is not None or self.record_last_noise
+        for group in self.param_groups:
+            if last_noise is None:
+                for p in group["params"]:
+                    if p.grad is None:
+                        continue
+                    param_state = self.state[p]
+                    if len(param_state) == 0:
+                        continue
+                    param_state["grad_sum"].add_(
+                        param_state["last_noise"]
+                    )  # add the last piece of noise to the current gradient sum
+            else:
+                for p, nz in zip(group["params"], last_noise):
+                    if p.grad is None:
+                        continue
+                    param_state = self.state[p]
+                    if len(param_state) == 0:
+                        continue
+                    param_state["grad_sum"].add_(nz)
+
+
 @dataclass
 class ServerOptimizerConfig:
     _target_: str = MISSING
@@ -259,63 +360,8 @@ class FedLAMBOptimizerConfig(ServerOptimizerConfig):
     eps: float = 1e-8
 
 
-class OptimizerType(Enum):
-    fed_avg: str = FedAvgOptimizerConfig._target_
-    fed_avg_with_lr: str = FedAvgWithLROptimizerConfig._target_
-    fed_adam: str = FedAdamOptimizerConfig._target_
-    fed_lamb: str = FedLAMBOptimizerConfig._target_
-    fed_lars: str = FedLARSOptimizerConfig._target_
-
-    @staticmethod
-    def create_optimizer(
-        model: nn.Module, config: ServerOptimizerConfig
-    ) -> IServerOptimizer:
-        if is_target(config, FedAvgWithLROptimizerConfig):
-            return torch.optim.SGD(
-                model.parameters(),
-                # pyre-ignore[16] Undefined attribute
-                lr=config.lr,
-                # pyre-ignore[16] Undefined attribute
-                momentum=config.momentum,
-            )
-        elif is_target(config, FedAdamOptimizerConfig):
-            return torch.optim.Adam(
-                model.parameters(),
-                lr=config.lr,
-                # pyre-ignore[16] Undefined attribute
-                weight_decay=config.weight_decay,
-                # pyre-ignore[16] Undefined attribute
-                betas=(config.beta1, config.beta2),
-                # pyre-ignore[16] Undefined attribute
-                eps=config.eps,
-            )
-        elif is_target(config, FedLARSOptimizerConfig):
-            # pyre-ignore[7]
-            return LARS(
-                model.parameters(),
-                lr=config.lr,
-                # pyre-ignore[16] Undefined attribute
-                beta=config.beta,
-                weight_decay=config.weight_decay,
-            )
-
-        elif is_target(config, FedLAMBOptimizerConfig):
-            # pyre-ignore[7]
-            return LAMB(
-                model.parameters(),
-                lr=config.lr,
-                beta1=config.beta1,
-                beta2=config.beta2,
-                weight_decay=config.weight_decay,
-                eps=config.eps,
-            )
-        elif is_target(config, FedAvgOptimizerConfig):
-            return torch.optim.SGD(
-                model.parameters(),
-                lr=1.0,
-                momentum=0,
-            )
-        else:
-            raise ValueError(
-                f"Optimizer type {config._target_} not found. Please update OptimizerType.create_optimizer"
-            )
+@dataclass
+class ServerFTRLOptimizerConfig(ServerOptimizerConfig):
+    _target_: str = fullclassname(ServerFTRLOptimizer)
+    lr: float = 0.001
+    momentum: float = 0.0
