@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -8,7 +9,6 @@ from hydra.core.config_store import ConfigStore  #  @manual
 
 from .base_round_reducer import RoundReducerConfig
 from .dp_round_reducer import DPRoundReducerConfig
-from .secure_round_reducer import SecureRoundReducerConfig
 from .weighted_dp_round_reducer import WeightedDPRoundReducerConfig
 
 
@@ -24,12 +24,6 @@ ConfigStore.instance().store(
     group="reducer",
 )
 
-ConfigStore.instance().store(
-    name="base_secure_reducer",
-    node=SecureRoundReducerConfig,
-    group="reducer",
-)
-
 ConfigStore.instance().store(
     name="base_weighted_dp_reducer",
     node=WeightedDPRoundReducerConfig,
