@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -27,8 +28,7 @@ from flsim.channels.base_channel import IdentityChannel
 from flsim.channels.message import Message
 from flsim.common.logger import Logger
 from flsim.interfaces.model import IFLModel
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
+from flsim.utils.config_utils import fullclassname, init_self_cfg
 from flsim.utils.distributed.fl_distributed import FLDistributedUtils, OperationType
 from flsim.utils.fl.common import FLModelParamUtils
 from omegaconf import MISSING
@@ -91,7 +91,7 @@ class IFLRoundReducer(abc.ABC):
     @abc.abstractmethod
     def collect_update(self, delta: IFLModel) -> None:
         """
-        Given a updated model from the client, add it to this reducer.
+        Given an updated model from the client, add it to this reducer.
         """
         pass
 
@@ -150,8 +150,8 @@ class RoundReducer(IFLRoundReducer):
         self.num_users_per_round = num_users_per_round
         self.total_number_of_users = total_number_of_users
         # TODO these are specific to mean reducer [this implementation]
-        # we will probably need a level of inheritence here and hide
-        # these frome the main class.
+        # we will probably need a level of inheritance here and hide
+        # these from the main class.
         self.sum_weights: torch.Tensor = torch.zeros(1)
         self.ref_model: IFLModel = global_model
         self.reduced_module: nn.Module = FLModelParamUtils.clone(
@@ -170,13 +170,13 @@ class RoundReducer(IFLRoundReducer):
 
     def _reduce_all(self, op: OperationType = OperationType.SUM_AND_BROADCAST):
         """
-        reduce models accross all workers if multi-processing is used.
+        Reduce models across all workers if multi-processing is used.
         reduction type is defined by `config.reduction_type`, see
         `ReductionType`.
 
         Returns:
             number of the models that have been collected. For weighted
-            reductions types returns sum of all model weights.
+            reduction types, returns the sum of all model weights.
 
         Note:
             The weights are sum of weights only for weighted reduction types
