@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -14,20 +15,16 @@ from flsim.channels.base_channel import IdentityChannel
 from flsim.interfaces.model import IFLModel
 from flsim.privacy.common import PrivacyBudget, PrivacySetting
 from flsim.privacy.privacy_engine import IPrivacyEngine
-from flsim.privacy.privacy_engine_factory import PrivacyEngineFactory, NoiseType
+from flsim.privacy.privacy_engine_factory import NoiseType, PrivacyEngineFactory
 from flsim.privacy.user_update_clip import UserUpdateClipper
-from flsim.reducers.secure_round_reducer import (
-    SecureRoundReducer,
-    SecureRoundReducerConfig,
-)
-from flsim.utils.config_utils import fullclassname
-from flsim.utils.config_utils import init_self_cfg
+from flsim.reducers.base_round_reducer import RoundReducer, RoundReducerConfig
+from flsim.utils.config_utils import fullclassname, init_self_cfg
 from flsim.utils.distributed.fl_distributed import FLDistributedUtils, OperationType
 from flsim.utils.fl.common import FLModelParamUtils
 from torch import nn
 
 
-class DPRoundReducer(SecureRoundReducer):
+class DPRoundReducer(RoundReducer):
     def __init__(
         self,
         *,
@@ -100,7 +97,7 @@ class DPRoundReducer(SecureRoundReducer):
                 self.logger.error(
                     f"total weights {total_weights} is not equal to "
                     f"number of users {self.num_users_per_round}. "
-                    "Please make sure reduction_type=AVERGAE."
+                    "Please make sure reduction_type=AVERAGE."
                 )
 
             """
@@ -141,7 +138,7 @@ class DPRoundReducer(SecureRoundReducer):
 
 
 @dataclass
-class DPRoundReducerConfig(SecureRoundReducerConfig):
+class DPRoundReducerConfig(RoundReducerConfig):
     """
     Contains configurations for a round reducer based on DP (user-level dp)
     """
