@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -12,27 +13,19 @@ from typing import Optional
 from flsim.active_user_selectors.simple_user_selector import (
     UniformlyRandomActiveUserSelectorConfig,
 )
-from flsim.channels.base_channel import (
-    IFLChannel,
-    IdentityChannel,
-)
+from flsim.channels.base_channel import IdentityChannel, IFLChannel
 from flsim.channels.message import Message
 from flsim.data.data_provider import IFLDataProvider
 from flsim.interfaces.model import IFLModel
-from flsim.optimizers.server_optimizers import (
-    FedAvgOptimizerConfig,
-)
-from flsim.privacy.common import PrivacySetting, PrivacyBudget
+from flsim.optimizers.server_optimizers import FedAvgOptimizerConfig
+from flsim.privacy.common import PrivacyBudget, PrivacySetting
 from flsim.privacy.privacy_engine import IPrivacyEngine
-from flsim.privacy.privacy_engine_factory import PrivacyEngineFactory, NoiseType
+from flsim.privacy.privacy_engine_factory import NoiseType, PrivacyEngineFactory
 from flsim.privacy.user_update_clip import UserUpdateClipper
 from flsim.servers.aggregator import AggregationType, Aggregator
-from flsim.servers.sync_servers import (
-    ISyncServer,
-    SyncServerConfig,
-)
+from flsim.servers.sync_servers import ISyncServer, SyncServerConfig
 from flsim.utils.config_utils import fullclassname, init_self_cfg
-from flsim.utils.distributed.fl_distributed import OperationType, FLDistributedUtils
+from flsim.utils.distributed.fl_distributed import FLDistributedUtils, OperationType
 from flsim.utils.fl.common import FLModelParamUtils
 from hydra.utils import instantiate
 from omegaconf import OmegaConf
@@ -139,7 +132,7 @@ class SyncDPSGDServer(ISyncServer):
         )
 
     def step(self):
-        assert self._privacy_engine is not None, "PrivacyEngine is not intialized"
+        assert self._privacy_engine is not None, "PrivacyEngine is not initialized"
 
         aggregated_model = self._aggregator.aggregate(distributed_op=OperationType.SUM)
 
@@ -149,10 +142,10 @@ class SyncDPSGDServer(ISyncServer):
                 self._clipping_value / self._aggregator.sum_weights.item(),
             )
 
-        FLDistributedUtils.synchronize_across_ranks(
+        FLDistributedUtils.synchronize_model_across_workers(
+            operation=OperationType.BROADCAST,
             model=aggregated_model,
             weights=self._aggregator.sum_weights,
-            operation=OperationType.BROADCAST,
         )
 
         FLModelParamUtils.set_gradient(
