@@ -1,51 +1,50 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
-import copy
 from enum import Enum
 from typing import Union
 
 import numpy as np
 import torch
-from flsim.common.pytest_helper import assertEqual, assertEmpty
+from flsim.common.pytest_helper import assertEmpty, assertEqual
 from flsim.data.data_provider import IFLDataProvider
 from flsim.interfaces.model import IFLModel
 from flsim.optimizers.async_aggregators import (
     AsyncAggregatorConfig,
-    FedAdamAsyncAggregatorConfig,
-    FedAvgWithLRAsyncAggregatorConfig,
-    FedAdamHybridAggregatorConfig,
-    FedAvgWithLRHybridAggregatorConfig,
     create_optimizer_for_async_aggregator,
+    FedAdamAsyncAggregatorConfig,
+    FedAdamFedBuffAggregatorConfig,
+    FedAvgWithLRAsyncAggregatorConfig,
+    FedAvgWithLRFedBuffAggregatorConfig,
 )
 from flsim.optimizers.server_optimizers import (
     FedAdamOptimizerConfig,
     FedAvgWithLROptimizerConfig,
-    OptimizerType,
-)
-from flsim.servers.sync_servers import (
-    SyncServerConfig,
-)
-from flsim.tests.utils import (
-    MetricsReporterWithMockedChannels,
-    verify_models_equivalent_after_training,
 )
+from flsim.servers.sync_servers import SyncServerConfig
 from flsim.trainers.async_trainer import AsyncTrainerConfig
 from flsim.trainers.sync_trainer import SyncTrainerConfig
 from flsim.utils.config_utils import is_target
+from flsim.utils.fl.common import FLModelParamUtils
 from flsim.utils.sample_model import DummyAlphabetFLModel
-from flsim.utils.tests.helpers.async_trainer_test_utils import (
+from flsim.utils.test_utils import (
+    MetricsReporterWithMockedChannels,
+    verify_models_equivalent_after_training,
+)
+from flsim.utils.tests.helpers.test_async_trainer_utils import (
     create_async_trainer,
     create_event_generator_config,
     get_safe_global_lr,
     run_fl_training,
 )
-from flsim.utils.tests.helpers.sync_trainer_test_utils import create_sync_trainer
 from flsim.utils.tests.helpers.test_data_utils import DummyAlphabetDataset
+from flsim.utils.tests.helpers.test_sync_trainer_utils import create_sync_trainer
 from flsim.utils.tests.helpers.test_utils import FLTestUtils
+from hydra.utils import instantiate
 
 
 class TrainerType(Enum):
@@ -54,7 +53,7 @@ class TrainerType(Enum):
     NONFL: str = "NonFL"
 
 
-class HybridFLTestUtils:
+class FedBuffTestUtils:
     @staticmethod
     def get_data_provider(
         num_examples: int,
@@ -75,18 +74,18 @@ class HybridFLTestUtils:
 
     @staticmethod
     def train_comparable_model(
-        trainer_to_compare_hybrid_fl_with,
+        trainer_to_compare_fedbuff_with,
         data_provider,
         global_model,
         server_config,
-        local_lr,
-        epochs,
-        training_rate,
-        training_duration_mean,
-        training_duration_sd,
+        local_lr: float,
+        epochs: int,
+        training_rate: int,
+        training_duration_mean: float,
+        training_duration_sd: float,
     ) -> IFLModel:
-        metric_reporter = MetricsReporterWithMockedChannels()
-        if trainer_to_compare_hybrid_fl_with == TrainerType.SYNC:
+        metrics_reporter = MetricsReporterWithMockedChannels()
+        if trainer_to_compare_fedbuff_with == TrainerType.SYNC:
             trainer = create_sync_trainer(
                 model=global_model,
                 local_lr=local_lr,
@@ -96,11 +95,11 @@ class HybridFLTestUtils:
             )
             model_to_compare, _ = trainer.train(
                 data_provider=data_provider,
-                metric_reporter=metric_reporter,
-                num_total_users=data_provider.num_users(),
+                metrics_reporter=metrics_reporter,
+                num_total_users=data_provider.num_train_users(),
                 distributed_world_size=1,
             )
-        elif trainer_to_compare_hybrid_fl_with == TrainerType.ASYNC:
+        elif trainer_to_compare_fedbuff_with == TrainerType.ASYNC:
             trainer = create_async_trainer(
                 model=global_model,
                 local_lr=local_lr,
@@ -114,11 +113,11 @@ class HybridFLTestUtils:
             )
             model_to_compare, _ = trainer.train(
                 data_provider=data_provider,
-                metric_reporter=metric_reporter,
-                num_total_users=data_provider.num_users(),
+                metrics_reporter=metrics_reporter,
+                num_total_users=data_provider.num_train_users(),
                 distributed_world_size=1,
             )
-        elif trainer_to_compare_hybrid_fl_with == TrainerType.NONFL:
+        elif trainer_to_compare_fedbuff_with == TrainerType.NONFL:
             # create an optimizer from aggregator_config.
             # for tests in this file, optimizer will either be
             # torch.optim.SGD or torch.optim.Adam
@@ -128,7 +127,7 @@ class HybridFLTestUtils:
                     model=global_model.fl_get_module(),
                 )
             elif isinstance(server_config, SyncServerConfig):
-                optimizer = OptimizerType.create_optimizer(
+                optimizer = instantiate(
                     model=global_model.fl_get_module(),
                     config=server_config.server_optimizer,
                 )
@@ -138,20 +137,20 @@ class HybridFLTestUtils:
                 data_provider=data_provider,
                 global_model=global_model,
                 optimizer=optimizer,
-                metrics_reporter=metric_reporter,
+                metrics_reporter=metrics_reporter,
                 epochs=epochs,
             )
         # pyre-fixme[61]: `model_to_compare` may not be initialized here.
         return model_to_compare
 
     @staticmethod
-    def get_hybrid_aggregator(
-        aggregator_config, buffer_size, hybrid_lr
-    ) -> Union[FedAdamHybridAggregatorConfig, FedAvgWithLRHybridAggregatorConfig]:
+    def get_fedbuff_aggregator(
+        aggregator_config, buffer_size, fedbuff_lr
+    ) -> Union[FedAdamFedBuffAggregatorConfig, FedAvgWithLRFedBuffAggregatorConfig]:
         if isinstance(aggregator_config, AsyncAggregatorConfig):
             if "FedAdam" in aggregator_config._target_:
-                hybrid_aggregator = FedAdamHybridAggregatorConfig(
-                    lr=hybrid_lr,
+                fedbuff_aggregator = FedAdamFedBuffAggregatorConfig(
+                    lr=fedbuff_lr,
                     # pyre-ignore[16]
                     weight_decay=aggregator_config.weight_decay,
                     # pyre-ignore[16]
@@ -159,17 +158,17 @@ class HybridFLTestUtils:
                     buffer_size=buffer_size,
                 )
             else:  # "FedAvgWithLR" in aggregator_config._target_:
-                hybrid_aggregator = FedAvgWithLRHybridAggregatorConfig(
-                    lr=hybrid_lr,
+                fedbuff_aggregator = FedAvgWithLRFedBuffAggregatorConfig(
+                    lr=fedbuff_lr,
                     # pyre-ignore[16]
                     momentum=aggregator_config.momentum,
                     buffer_size=buffer_size,
                 )
-            return hybrid_aggregator
+            return fedbuff_aggregator
         elif isinstance(aggregator_config, SyncServerConfig):
             if is_target(aggregator_config.server_optimizer, FedAdamOptimizerConfig):
-                hybrid_aggregator = FedAdamHybridAggregatorConfig(
-                    lr=hybrid_lr,
+                fedbuff_aggregator = FedAdamFedBuffAggregatorConfig(
+                    lr=fedbuff_lr,
                     # pyre-ignore[16]
                     weight_decay=aggregator_config.server_optimizer.weight_decay,
                     # pyre-ignore[16]
@@ -177,42 +176,42 @@ class HybridFLTestUtils:
                     buffer_size=buffer_size,
                 )
             else:
-                hybrid_aggregator = FedAvgWithLRHybridAggregatorConfig(
-                    lr=hybrid_lr,
+                fedbuff_aggregator = FedAvgWithLRFedBuffAggregatorConfig(
+                    lr=fedbuff_lr,
                     # pyre-ignore[16]
                     momentum=aggregator_config.server_optimizer.momentum,
                     buffer_size=buffer_size,
                 )
-            return hybrid_aggregator
+            return fedbuff_aggregator
         else:
             raise ValueError("Invalid config", aggregator_config)
 
     @staticmethod
-    def compare_hybrid_fl_same(
-        trainer_to_compare_hybrid_fl_with,
+    def compare_fedbuff_same(
+        trainer_to_compare_fedbuff_with,
         trainer_to_compare_aggregator_config,
-        hybrid_aggregator_config,
-        base_local_lr,
-        hybrid_local_lr,
-        epochs,
-        num_examples,
-        num_fl_users,
-        batch_size,
-        examples_per_user,
+        fedbuff_aggregator_config,
+        base_local_lr: float,
+        fedbuff_local_lr: float,
+        epochs: int,
+        num_examples: int,
+        num_fl_users: int,
+        batch_size: int,
+        examples_per_user: int,
         buffer_size,
-        training_rate,
-        training_duration_mean,
-        training_duration_sd,
+        training_rate: int,
+        training_duration_mean: float,
+        training_duration_sd: float,
     ) -> str:
         # we need to make three copies:
         # to train the model we want to compare with
         global_model = DummyAlphabetFLModel()
-        # to train hybrid model
-        global_model_hybrid_copy = copy.deepcopy(global_model)
+        # to train a model with FedBuff
+        global_model_fedbuff_copy = FLModelParamUtils.clone(global_model)
         # to verify training indeed took place
-        global_model_init_copy = copy.deepcopy(global_model)
+        global_model_init_copy = FLModelParamUtils.clone(global_model)
 
-        data_provider = HybridFLTestUtils.get_data_provider(
+        data_provider = FedBuffTestUtils.get_data_provider(
             num_examples=num_examples,
             num_fl_users=num_fl_users,
             examples_per_user=examples_per_user,
@@ -221,8 +220,8 @@ class HybridFLTestUtils:
         )
 
         def get_base_trained_model():
-            model_to_compare = HybridFLTestUtils.train_comparable_model(
-                trainer_to_compare_hybrid_fl_with=trainer_to_compare_hybrid_fl_with,
+            model_to_compare = FedBuffTestUtils.train_comparable_model(
+                trainer_to_compare_fedbuff_with=trainer_to_compare_fedbuff_with,
                 data_provider=data_provider,
                 global_model=global_model,
                 server_config=trainer_to_compare_aggregator_config,
@@ -234,33 +233,33 @@ class HybridFLTestUtils:
             )
             return model_to_compare
 
-        def get_hybrid_trained_model():
-            hybrid_fl_trainer = create_async_trainer(
-                model=global_model_hybrid_copy,
-                local_lr=hybrid_local_lr,
+        def get_fedbuff_trained_model():
+            fedbuff_trainer = create_async_trainer(
+                model=global_model_fedbuff_copy,
+                local_lr=fedbuff_local_lr,
                 epochs=epochs,
                 event_generator_config=create_event_generator_config(
                     training_rate=training_rate,
                     training_duration_mean=training_duration_mean,
                     training_duration_sd=training_duration_sd,
                 ),
-                aggregator_config=hybrid_aggregator_config,
+                aggregator_config=fedbuff_aggregator_config,
             )
 
-            hybrid_model, _ = hybrid_fl_trainer.train(
+            fedbuff_model, _ = fedbuff_trainer.train(
                 data_provider=data_provider,
-                metric_reporter=MetricsReporterWithMockedChannels(),
-                num_total_users=data_provider.num_users(),
+                metrics_reporter=MetricsReporterWithMockedChannels(),
+                num_total_users=data_provider.num_train_users(),
                 distributed_world_size=1,
             )
-            return hybrid_model
+            return fedbuff_model
 
-        hybrid_trained_model = get_hybrid_trained_model()
+        fedbuff_trained_model = get_fedbuff_trained_model()
         base_model = get_base_trained_model()
 
         error_msg = verify_models_equivalent_after_training(
             base_model,
-            hybrid_trained_model,
+            fedbuff_trained_model,
             global_model_init_copy,
             rel_epsilon=1e-4,
             abs_epsilon=1e-6,
@@ -285,27 +284,27 @@ class HybridFLTestUtils:
         return num_fl_users, examples_per_user, num_examples, training_rate
 
     @staticmethod
-    def compare_nonfl_hybrid_uneven_data_split(
-        total_examples,
-        num_fl_users,
+    def compare_nonfl_fedbuff_uneven_data_split(
+        total_examples: int,
+        num_fl_users: int,
         buffer_size,
         non_fl_lr,
-        hybrid_global_lr,
-        examples_per_user_hybrid,
-        examples_per_user_nonfl,
-        batch_size_hybrid,
-        batch_size_nonfl,
-        epochs,
-        local_lr=1.0,
-    ):
+        fedbuff_global_lr,
+        examples_per_user_fedbuff: int,
+        examples_per_user_nonfl: int,
+        batch_size_fedbuff: int,
+        batch_size_nonfl: int,
+        epochs: int,
+        local_lr: float = 1.0,
+    ) -> str:
         # to verify training indeed took place
         reference_untrained_model = DummyAlphabetFLModel()
-        # to train hybrid model
-        hybrid_model = copy.deepcopy(reference_untrained_model)
+        # to train a model with FedVuff
+        fedbuff_model = FLModelParamUtils.clone(reference_untrained_model)
         # to train nonfl
-        nonfl_model = copy.deepcopy(reference_untrained_model)
+        nonfl_model = FLModelParamUtils.clone(reference_untrained_model)
 
-        nonfl_data_provider = HybridFLTestUtils.get_data_provider(
+        nonfl_data_provider = FedBuffTestUtils.get_data_provider(
             num_examples=total_examples,
             num_fl_users=1,
             examples_per_user=examples_per_user_nonfl,
@@ -313,21 +312,21 @@ class HybridFLTestUtils:
             model=nonfl_model,
         )
 
-        hybrid_data_provider = HybridFLTestUtils.get_data_provider(
+        fedbuff_data_provider = FedBuffTestUtils.get_data_provider(
             num_examples=total_examples,
             num_fl_users=num_fl_users,
-            examples_per_user=examples_per_user_hybrid,
-            batch_size=batch_size_hybrid,
-            model=hybrid_model,
+            examples_per_user=examples_per_user_fedbuff,
+            batch_size=batch_size_fedbuff,
+            model=fedbuff_model,
         )
 
-        hybrid_trained_model, _ = run_fl_training(
-            fl_model=hybrid_model,
-            fl_data_provider=hybrid_data_provider,
+        fedbuff_trained_model, _ = run_fl_training(
+            fl_model=fedbuff_model,
+            fl_data_provider=fedbuff_data_provider,
             epochs=epochs,
             local_lr=local_lr,
-            aggregator_config=FedAvgWithLRHybridAggregatorConfig(
-                lr=hybrid_global_lr, buffer_size=buffer_size
+            aggregator_config=FedAvgWithLRFedBuffAggregatorConfig(
+                lr=fedbuff_global_lr, buffer_size=buffer_size
             ),
             # sequential training training_rate >> training_duration
             training_rate=1,
@@ -343,37 +342,39 @@ class HybridFLTestUtils:
             optimizer=optimizer,
             epochs=epochs,
         )
-        print(f"Local LR {local_lr} Non LR {non_fl_lr} Hybrid LR {hybrid_global_lr}")
+        print(
+            f"Local LR {local_lr} Non FL LR {non_fl_lr} FedBuff LR {fedbuff_global_lr}"
+        )
         return verify_models_equivalent_after_training(
             nonfl_trained_model,
-            hybrid_trained_model,
+            fedbuff_trained_model,
             reference_untrained_model,
             rel_epsilon=1e-4,
             abs_epsilon=1e-6,
         )
 
     @staticmethod
-    def compare_async_hybrid_uneven_data_split(
-        total_examples,
-        num_fl_users,
-        hybrid_num_fl_users,
+    def compare_async_fedbuff_uneven_data_split(
+        total_examples: int,
+        num_fl_users: int,
+        fedbuff_num_fl_users,
         async_global_lr,
-        hybrid_global_lr,
-        batch_size_hybrid,
-        batch_size_async,
-        epochs,
-        training_rate,
-        training_duration_mean,
-        local_lr=1.0,
-    ):
+        fedbuff_global_lr,
+        batch_size_fedbuff: int,
+        batch_size_async: int,
+        epochs: int,
+        training_rate: int,
+        training_duration_mean: float,
+        local_lr: float = 1.0,
+    ) -> str:
         # to verify training indeed took place
         reference_untrained_model = DummyAlphabetFLModel()
-        # to train hybrid model
-        hybrid_model = copy.deepcopy(reference_untrained_model)
+        # to train a model with FedBuff
+        fedbuff_model = FLModelParamUtils.clone(reference_untrained_model)
         # to train async
-        async_model = copy.deepcopy(reference_untrained_model)
+        async_model = FLModelParamUtils.clone(reference_untrained_model)
 
-        async_data_provider = HybridFLTestUtils.get_data_provider(
+        async_data_provider = FedBuffTestUtils.get_data_provider(
             num_examples=total_examples,
             num_fl_users=1,
             examples_per_user=total_examples,
@@ -390,30 +391,30 @@ class HybridFLTestUtils:
             training_duration_mean=training_duration_mean,
         )
 
-        hybrid_data_provider = HybridFLTestUtils.get_data_provider(
+        fedbuff_data_provider = FedBuffTestUtils.get_data_provider(
             num_examples=total_examples,
             num_fl_users=num_fl_users,
             examples_per_user=total_examples // num_fl_users,
-            batch_size=batch_size_hybrid,
-            model=hybrid_model,
+            batch_size=batch_size_fedbuff,
+            model=fedbuff_model,
         )
-        hybrid_trained_model, _ = run_fl_training(
-            fl_model=hybrid_model,
-            fl_data_provider=hybrid_data_provider,
+        fedbuff_trained_model, _ = run_fl_training(
+            fl_model=fedbuff_model,
+            fl_data_provider=fedbuff_data_provider,
             epochs=epochs,
             local_lr=local_lr,
-            aggregator_config=FedAvgWithLRHybridAggregatorConfig(
-                lr=hybrid_global_lr, buffer_size=hybrid_num_fl_users
+            aggregator_config=FedAvgWithLRFedBuffAggregatorConfig(
+                lr=fedbuff_global_lr, buffer_size=fedbuff_num_fl_users
             ),
             training_rate=training_rate,
             training_duration_mean=training_duration_mean,
         )
         print(
-            f"Local LR {local_lr} Async LR {async_global_lr} Hybrid LR {hybrid_global_lr}"
+            f"Local LR {local_lr} Async LR {async_global_lr} FedBuff LR {fedbuff_global_lr}"
         )
         return verify_models_equivalent_after_training(
             async_trained_model,
-            hybrid_trained_model,
+            fedbuff_trained_model,
             reference_untrained_model,
             rel_epsilon=1e-4,
             abs_epsilon=1e-6,
@@ -422,15 +423,15 @@ class HybridFLTestUtils:
 
 class TestFedBuff:
     @classmethod
-    def setup_class(cls):
+    def setup_class(cls) -> None:
         np.random.seed(0)
 
-    def test_async_hybrid_same_multiple_clients_to_sync(self):
+    def test_async_fedbuff_same_multiple_clients_to_sync(self) -> None:
         r"""
-        Test when hybrid has multiple clients to sync
+        Test when FedBuff has multiple clients to sync
         1. Training in parallel. Meaning, both mechanisms start from the same global model (training_time >> training rate)
         2. Total examples is the same for both and local lr is 1.0
-        3. Pure Async 1 user with 10 examples should be the same as hybrid 10 users each with 1 example
+        3. Pure Async 1 user with 10 examples should be the same as FedBuff 10 users each with 1 example
         """
         # training in parallel training_duration >> training_rate
         training_rate = 1
@@ -438,20 +439,20 @@ class TestFedBuff:
 
         total_examples = 10
         num_fl_users = 10
-        batch_size_hybrid = 1
+        batch_size_fedbuff = 1
         batch_size_async = total_examples
 
         buffer_size = num_fl_users
         local_lr = np.random.sample()
         async_global_lr = np.random.sample()
-        hybrid_global_lr = async_global_lr / buffer_size
-        error_msg = HybridFLTestUtils.compare_async_hybrid_uneven_data_split(
+        fedbuff_global_lr = async_global_lr / buffer_size
+        error_msg = FedBuffTestUtils.compare_async_fedbuff_uneven_data_split(
             total_examples=total_examples,
             num_fl_users=num_fl_users,
-            hybrid_num_fl_users=buffer_size,
+            fedbuff_num_fl_users=buffer_size,
             async_global_lr=async_global_lr,
-            hybrid_global_lr=hybrid_global_lr,
-            batch_size_hybrid=batch_size_hybrid,
+            fedbuff_global_lr=fedbuff_global_lr,
+            batch_size_fedbuff=batch_size_fedbuff,
             batch_size_async=batch_size_async,
             epochs=5,
             training_rate=training_rate,
@@ -460,48 +461,48 @@ class TestFedBuff:
         )
         assertEmpty(error_msg, msg=error_msg)
 
-    def test_non_fl_hybrid_same_multiple_clients_to_sync(self):
+    def test_non_fl_fedbuff_same_multiple_clients_to_sync(self) -> None:
         """
-        Test nonFL and hybrid same with multiple clients to sync
+        Test nonFL and FedBuff same with multiple clients to sync
 
         Both mechanisms train on the number of examples yield the same result if
-        1. hybrid is trained sequentially (training rate >> training duration)
-        2. NonFL has 1 user with 10 examples and hybrid has 10 users each with 1 example trained for 1 epoch
+        1. FedBuff is trained sequentially (training rate >> training duration)
+        2. NonFL has 1 user with 10 examples and FedBuff has 10 users each with 1 example trained for 1 epoch
         3. Local_lr = some random value, global_lr = some random value
-        4. Hybrid_global_lr = global_lr / (buffer_size * local_lr)
+        4. fedbuff_global_lr = global_lr / (buffer_size * local_lr)
         """
         total_examples = 10
         num_fl_users = 10
-        examples_per_user_hybrid = 1
-        batch_size_hybrid = 1
+        examples_per_user_fedbuff = 1
+        batch_size_fedbuff = 1
         batch_size_nonfl = total_examples
 
         buffer_size = 10
         nonfl_lr = np.random.sample()
-        hybrid_local_lr = np.random.sample()
-        hybrid_global_lr = nonfl_lr / (buffer_size * hybrid_local_lr)
+        fedbuff_local_lr = np.random.sample()
+        fedbuff_global_lr = nonfl_lr / (buffer_size * fedbuff_local_lr)
 
-        error_msg = HybridFLTestUtils.compare_nonfl_hybrid_uneven_data_split(
+        error_msg = FedBuffTestUtils.compare_nonfl_fedbuff_uneven_data_split(
             total_examples=total_examples,
             num_fl_users=num_fl_users,
-            examples_per_user_hybrid=examples_per_user_hybrid,
+            examples_per_user_fedbuff=examples_per_user_fedbuff,
             examples_per_user_nonfl=total_examples,
-            batch_size_hybrid=batch_size_hybrid,
+            batch_size_fedbuff=batch_size_fedbuff,
             batch_size_nonfl=batch_size_nonfl,
             buffer_size=buffer_size,
             non_fl_lr=nonfl_lr,
-            hybrid_global_lr=hybrid_global_lr,
-            local_lr=hybrid_local_lr,
+            fedbuff_global_lr=fedbuff_global_lr,
+            local_lr=fedbuff_local_lr,
             epochs=1,
         )
         assertEmpty(error_msg, msg=error_msg)
 
-    def test_async_hybrid_same_sync_every_client(self):
+    def test_async_fedbuff_same_sync_every_client(self) -> None:
         r"""
-        Hybrid and Async should yield the same model:
-        1. Async and Hybrid training duration distribution are the same
+        FedBuff and Async should yield the same model:
+        1. Async and FedBuff training duration distribution are the same
         training_duration_sd should be small
-        2. Hybrid update global model every 1 client
+        2. FedBuff update global model every 1 client
         """
         # random learning rate between 0 and 10
         local_lr = np.random.sample()
@@ -517,21 +518,21 @@ class TestFedBuff:
                     examples_per_user,
                     num_examples,
                     training_rate,
-                ) = HybridFLTestUtils.get_data_params(
+                ) = FedBuffTestUtils.get_data_params(
                     min_num_users=2,
                     max_num_users=10,
                     min_examples_per_user=1,
                     max_examples_per_user=10,
                 )
-                error_msg = HybridFLTestUtils.compare_hybrid_fl_same(
-                    trainer_to_compare_hybrid_fl_with=TrainerType.ASYNC,
+                error_msg = FedBuffTestUtils.compare_fedbuff_same(
+                    trainer_to_compare_fedbuff_with=TrainerType.ASYNC,
                     trainer_to_compare_aggregator_config=base_aggregator_config,
-                    hybrid_aggregator_config=HybridFLTestUtils.get_hybrid_aggregator(
+                    fedbuff_aggregator_config=FedBuffTestUtils.get_fedbuff_aggregator(
                         aggregator_config=base_aggregator_config,
                         buffer_size=buffer_size,
-                        hybrid_lr=global_lr,
+                        fedbuff_lr=global_lr,
                     ),
-                    hybrid_local_lr=local_lr,
+                    fedbuff_local_lr=local_lr,
                     base_local_lr=local_lr,
                     epochs=5,
                     num_examples=num_examples,
@@ -545,19 +546,19 @@ class TestFedBuff:
                 )
                 assertEqual(error_msg, "")
 
-    def test_nonfl_hybrid_same_sgd(self):
+    def test_nonfl_fedbuff_same_sgd(self) -> None:
         r"""
-        Hybrid and NonFL should yield the same model:
-        1. Hybrid's training rate = 1, training duration ~ N(0, 0)
+        FedBuff and NonFL should yield the same model:
+        1. FedBuff's training rate = 1, training duration ~ N(0, 0)
         2. Round robin user selector
-        3. Hybrid takes global step  every 1 clients
+        3. FedBuff takes global step  every 1 clients
         """
         (
             num_fl_users,
             examples_per_user,
             num_examples,
             _,
-        ) = HybridFLTestUtils.get_data_params(
+        ) = FedBuffTestUtils.get_data_params(
             min_num_users=1,
             max_num_users=10,
             min_examples_per_user=1,
@@ -570,15 +571,15 @@ class TestFedBuff:
             base_aggregator_config = SyncServerConfig(
                 server_optimizer=FedAvgWithLROptimizerConfig(lr=global_lr, momentum=0.0)
             )
-            error = HybridFLTestUtils.compare_hybrid_fl_same(
-                trainer_to_compare_hybrid_fl_with=TrainerType.NONFL,
+            error = FedBuffTestUtils.compare_fedbuff_same(
+                trainer_to_compare_fedbuff_with=TrainerType.NONFL,
                 trainer_to_compare_aggregator_config=base_aggregator_config,
-                hybrid_aggregator_config=HybridFLTestUtils.get_hybrid_aggregator(
+                fedbuff_aggregator_config=FedBuffTestUtils.get_fedbuff_aggregator(
                     aggregator_config=base_aggregator_config,
                     buffer_size=buffer_size,
-                    hybrid_lr=global_lr,
+                    fedbuff_lr=global_lr,
                 ),
-                hybrid_local_lr=local_lr,
+                fedbuff_local_lr=local_lr,
                 base_local_lr=local_lr,
                 epochs=1,
                 num_examples=num_examples,
@@ -592,19 +593,19 @@ class TestFedBuff:
             )
             assertEqual(error, "")
 
-    def test_nonfl_hybrid_same_adam(self):
+    def test_nonfl_fedbuff_same_adam(self) -> None:
         r"""
-        Hybrid and NonFL should yield the same model:
-        1. Hybrid's training rate = 1, training duration ~ N(0, 0)
+        FedBuff and NonFL should yield the same model:
+        1. FedBuff's training rate = 1, training duration ~ N(0, 0)
         2. Round robin user selector
-        3. Hybrid takes global step  every 1 clients
+        3. FedBuff takes global step  every 1 clients
         """
         (
             num_fl_users,
             examples_per_user,
             num_examples,
             _,
-        ) = HybridFLTestUtils.get_data_params(
+        ) = FedBuffTestUtils.get_data_params(
             min_num_users=1,
             max_num_users=10,
             min_examples_per_user=1,
@@ -616,15 +617,15 @@ class TestFedBuff:
             server_optimizer=FedAdamOptimizerConfig(lr=global_lr, eps=1e-2)
         )
         buffer_size = 1
-        error = HybridFLTestUtils.compare_hybrid_fl_same(
-            trainer_to_compare_hybrid_fl_with=TrainerType.NONFL,
+        error = FedBuffTestUtils.compare_fedbuff_same(
+            trainer_to_compare_fedbuff_with=TrainerType.NONFL,
             trainer_to_compare_aggregator_config=base_aggregator_config,
-            hybrid_aggregator_config=HybridFLTestUtils.get_hybrid_aggregator(
+            fedbuff_aggregator_config=FedBuffTestUtils.get_fedbuff_aggregator(
                 aggregator_config=base_aggregator_config,
                 buffer_size=buffer_size,
-                hybrid_lr=global_lr,
+                fedbuff_lr=global_lr,
             ),
-            hybrid_local_lr=local_lr,
+            fedbuff_local_lr=local_lr,
             base_local_lr=local_lr,
             epochs=5,
             num_examples=num_examples,
@@ -638,24 +639,24 @@ class TestFedBuff:
         )
         assertEqual(error, "")
 
-    def test_sync_hybrid_same_sgd(self):
+    def test_sync_fedbuff_same_sgd(self) -> None:
         r"""
-        Hybrid and Sync should yield the same model:
+        FedBuff and Sync should yield the same model:
         1. For simplification, assume training_rate = total_users.
-        Without this constraint, Hybrid==Sync only for 1 round of sync.
+        Without this constraint, FedBuff==Sync only for 1 round of sync.
         2. `(Training_rate*num_users) << training_duration_mean`.
         E.g, `#users=10, training rate = 1, training_duration_mean > 10`.
         This is needed so that all users train in parallel.
         In particular, in a single epoch, every user starts with the same initial model
         3. Round robin user selector
-        4. Set hybrid_global_lr = sync_global_lr / num_clients_sync
+        4. Set fedbuff_global_lr = sync_global_lr / num_clients_sync
         """
         (
             num_fl_users,
             examples_per_user,
             num_examples,
             _,
-        ) = HybridFLTestUtils.get_data_params(
+        ) = FedBuffTestUtils.get_data_params(
             min_num_users=1,
             max_num_users=5,
             min_examples_per_user=1,
@@ -669,15 +670,15 @@ class TestFedBuff:
         buffer_size = num_fl_users
         for batch_size in [4, 16, 32]:
             print(f"{num_fl_users} {examples_per_user} {local_lr} {global_lr}")
-            error = HybridFLTestUtils.compare_hybrid_fl_same(
-                trainer_to_compare_hybrid_fl_with=TrainerType.SYNC,
+            error = FedBuffTestUtils.compare_fedbuff_same(
+                trainer_to_compare_fedbuff_with=TrainerType.SYNC,
                 trainer_to_compare_aggregator_config=base_aggregator_config,
-                hybrid_aggregator_config=HybridFLTestUtils.get_hybrid_aggregator(
+                fedbuff_aggregator_config=FedBuffTestUtils.get_fedbuff_aggregator(
                     aggregator_config=base_aggregator_config,
                     buffer_size=buffer_size,
-                    hybrid_lr=global_lr / buffer_size,
+                    fedbuff_lr=global_lr / buffer_size,
                 ),
-                hybrid_local_lr=local_lr,
+                fedbuff_local_lr=local_lr,
                 base_local_lr=local_lr,
                 epochs=1,
                 num_examples=num_examples,
@@ -691,28 +692,28 @@ class TestFedBuff:
             )
             assertEqual(error, "")
 
-    def test_sync_hybrid_same_adam(self):
+    def test_sync_fedbuff_same_adam(self) -> None:
         r"""
-        For sync == hybrid adam,
-        `hybrid_local_lr = sync_local_lr / buffer_size`
-        and hybrid and sync global lr's should be the same.
+        For sync == FedBuff adam,
+        `fedbuff_local_lr = sync_local_lr / buffer_size`
+        and FedBuff and sync global lr's should be the same.
 
-        Sync and hybrid compute different "delta"
-        hybrid_delta = sync_delta * buffer_size
-        to fix this difference, for SGD, we set `global_lr_hybrid = global_lr_sync/buffer_size`
+        Sync and FedBuff compute different "delta"
+        fedbuff_delta = sync_delta * buffer_size
+        to fix this difference, for SGD, we set `global_lr_fedbuff = global_lr_sync/buffer_size`
         however, for Adam, this normalization doesn't work:
         Since Adam stores first and second moments (mean and variance) of deltas.
         In particular, the following are not equivalent:
         delta=d, lr=l
         delta=d*k, lr=l/k
-        instead, we have to set `local_lr_hybrid = local_lr_sync / buffer_size`
+        instead, we have to set `local_lr_fedbuff = local_lr_sync / buffer_size`
         """
         (
             num_fl_users,
             examples_per_user,
             num_examples,
             _,
-        ) = HybridFLTestUtils.get_data_params(
+        ) = FedBuffTestUtils.get_data_params(
             min_num_users=1,
             max_num_users=10,
             min_examples_per_user=1,
@@ -725,15 +726,15 @@ class TestFedBuff:
             server_optimizer=FedAdamOptimizerConfig(lr=global_lr, eps=1e-2)
         )
         buffer_size = num_fl_users
-        error = HybridFLTestUtils.compare_hybrid_fl_same(
-            trainer_to_compare_hybrid_fl_with=TrainerType.SYNC,
+        error = FedBuffTestUtils.compare_fedbuff_same(
+            trainer_to_compare_fedbuff_with=TrainerType.SYNC,
             trainer_to_compare_aggregator_config=base_aggregator_config,
-            hybrid_aggregator_config=HybridFLTestUtils.get_hybrid_aggregator(
+            fedbuff_aggregator_config=FedBuffTestUtils.get_fedbuff_aggregator(
                 aggregator_config=base_aggregator_config,
                 buffer_size=buffer_size,
-                hybrid_lr=global_lr,
+                fedbuff_lr=global_lr,
             ),
-            hybrid_local_lr=local_lr / num_fl_users,
+            fedbuff_local_lr=local_lr / num_fl_users,
             base_local_lr=local_lr,
             epochs=1,
             num_examples=num_examples,
@@ -748,7 +749,7 @@ class TestFedBuff:
         print(f"{num_fl_users} {examples_per_user} {local_lr} {global_lr}")
         assertEqual(error, "")
 
-    def test_partial_model_update(self):
+    def test_partial_model_update(self) -> None:
         r"""
         Test for partial update
 
@@ -765,34 +766,34 @@ class TestFedBuff:
         """
         num_fl_users = 2
         total_examples = 20
-        examples_per_user_hybrid = total_examples // num_fl_users
+        examples_per_user_fedbuff = total_examples // num_fl_users
         buffer_size = num_fl_users
 
         nonfl_lr = np.random.sample()
-        hybrid_local_lr = np.random.sample()
+        fedbuff_local_lr = np.random.sample()
         # We need to normalize global_lr to account for the local training
-        # hence we need to divide by the (buffer_size * hybrid_local_lr)
-        hybrid_global_lr = nonfl_lr / (buffer_size * hybrid_local_lr)
+        # hence we need to divide by the (buffer_size * fedbuff_local_lr)
+        fedbuff_global_lr = nonfl_lr / (buffer_size * fedbuff_local_lr)
 
-        error_msg = HybridFLTestUtils.compare_nonfl_hybrid_uneven_data_split(
+        error_msg = FedBuffTestUtils.compare_nonfl_fedbuff_uneven_data_split(
             total_examples=total_examples,
             num_fl_users=num_fl_users,
             buffer_size=buffer_size,
             non_fl_lr=nonfl_lr,
-            hybrid_global_lr=hybrid_global_lr,
-            local_lr=hybrid_local_lr,
-            examples_per_user_hybrid=examples_per_user_hybrid,
+            fedbuff_global_lr=fedbuff_global_lr,
+            local_lr=fedbuff_local_lr,
+            examples_per_user_fedbuff=examples_per_user_fedbuff,
             examples_per_user_nonfl=total_examples,
-            batch_size_hybrid=examples_per_user_hybrid,
+            batch_size_fedbuff=examples_per_user_fedbuff,
             batch_size_nonfl=total_examples,
             epochs=1,
         )
         print(
-            f"NonFL LR {nonfl_lr} Hybrid Local LR {hybrid_local_lr}  Hybrid Global LR {hybrid_global_lr}"
+            f"NonFL LR {nonfl_lr} FedBuff Local LR {fedbuff_local_lr}  FedBuff Global LR {fedbuff_global_lr}"
         )
         assertEmpty(error_msg, msg=error_msg)
 
-    def test_remaining_clients_to_sync(self):
+    def test_remaining_clients_to_sync(self) -> None:
         """
         An aggregator can have unaggregated clients before
         an end of the epoch
@@ -807,7 +808,7 @@ class TestFedBuff:
 
         total_examples = 10
         num_fl_users = 10
-        batch_size_hybrid = 1
+        batch_size_fedbuff = 1
         batch_size_async = total_examples
 
         # test for scenario 1
@@ -815,14 +816,14 @@ class TestFedBuff:
         local_lr = np.random.sample()
         async_global_lr = np.random.sample()
 
-        hybrid_global_lr = async_global_lr / num_fl_users
-        error_msg = HybridFLTestUtils.compare_async_hybrid_uneven_data_split(
+        fedbuff_global_lr = async_global_lr / num_fl_users
+        error_msg = FedBuffTestUtils.compare_async_fedbuff_uneven_data_split(
             total_examples=total_examples,
             num_fl_users=num_fl_users,
-            hybrid_num_fl_users=buffer_size,
+            fedbuff_num_fl_users=buffer_size,
             async_global_lr=async_global_lr,
-            hybrid_global_lr=hybrid_global_lr,
-            batch_size_hybrid=batch_size_hybrid,
+            fedbuff_global_lr=fedbuff_global_lr,
+            batch_size_fedbuff=batch_size_fedbuff,
             batch_size_async=batch_size_async,
             epochs=1,
             training_rate=training_rate,
@@ -833,14 +834,14 @@ class TestFedBuff:
 
         # test for scenario 2
         buffer_size = 4
-        hybrid_global_lr = async_global_lr / num_fl_users
-        error_msg = HybridFLTestUtils.compare_async_hybrid_uneven_data_split(
+        fedbuff_global_lr = async_global_lr / num_fl_users
+        error_msg = FedBuffTestUtils.compare_async_fedbuff_uneven_data_split(
             total_examples=total_examples,
             num_fl_users=num_fl_users,
-            hybrid_num_fl_users=buffer_size,
+            fedbuff_num_fl_users=buffer_size,
             async_global_lr=async_global_lr,
-            hybrid_global_lr=hybrid_global_lr,
-            batch_size_hybrid=batch_size_hybrid,
+            fedbuff_global_lr=fedbuff_global_lr,
+            batch_size_fedbuff=batch_size_fedbuff,
             batch_size_async=batch_size_async,
             epochs=1,
             training_rate=training_rate,
