@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
@@ -18,47 +19,43 @@ from flsim.active_user_selectors.simple_user_selector import (
 )
 from flsim.clients.base_client import ClientConfig
 from flsim.common.pytest_helper import (
-    assertEqual,
-    assertTrue,
-    assertIsInstance,
     assertEmpty,
+    assertEqual,
+    assertIsInstance,
+    assertTrue,
 )
 from flsim.common.timeout_simulator import GaussianTimeOutSimulatorConfig
 from flsim.data.data_provider import FLDataProviderFromList
 from flsim.data.data_sharder import SequentialSharder
 from flsim.data.dataset_data_loader import FLDatasetDataLoaderWithBatch
-from flsim.fb.data.hive_data_utils import create_dataloader
-from flsim.fb.data.hive_dataset import InlineDatasetConfig
-from flsim.fb.data.paged_dataloader import PagedDataProvider
 from flsim.interfaces.metrics_reporter import TrainingStage
 from flsim.optimizers.async_aggregators import FedAdamAsyncAggregatorConfig
 from flsim.optimizers.local_optimizers import (
-    LocalOptimizerSGDConfig,
     LocalOptimizerFedProxConfig,
+    LocalOptimizerSGDConfig,
 )
 from flsim.optimizers.optimizer_scheduler import ArmijoLineSearchSchedulerConfig
 from flsim.optimizers.server_optimizers import (
     FedAdamOptimizerConfig,
     FedAvgWithLROptimizerConfig,
 )
-from flsim.servers.sync_servers import (
-    SyncServerConfig,
-)
-from flsim.tests.utils import (
-    FakeMetricReporter,
-    MockRecord,
-    MetricsReporterWithMockedChannels,
-    SimpleMetricReporter,
-    verify_models_equivalent_after_training,
-)
-from flsim.tests.utils import SampleNetHive
+from flsim.secure_aggregation.secure_aggregator import FixedPointConfig
+from flsim.servers.sync_secagg_servers import SyncSecAggServerConfig
+from flsim.servers.sync_servers import SyncServerConfig
 from flsim.trainers.async_trainer import AsyncTrainer, AsyncTrainerConfig
 from flsim.trainers.sync_trainer import SyncTrainer, SyncTrainerConfig
 from flsim.utils.config_utils import fl_config_from_json
 from flsim.utils.fl.common import FLModelParamUtils
 from flsim.utils.sample_model import DummyAlphabetFLModel
-from flsim.utils.tests.helpers.sync_trainer_test_utils import create_sync_trainer
+from flsim.utils.test_utils import (
+    FakeMetricReporter,
+    MetricsReporterWithMockedChannels,
+    MockRecord,
+    SimpleMetricReporter,
+    verify_models_equivalent_after_training,
+)
 from flsim.utils.tests.helpers.test_data_utils import DummyAlphabetDataset
+from flsim.utils.tests.helpers.test_sync_trainer_utils import create_sync_trainer
 from flsim.utils.tests.helpers.test_utils import FLTestUtils
 from flsim.utils.timing.training_duration_distribution import (
     PerExampleGaussianDurationDistributionConfig,
@@ -94,7 +91,7 @@ class TestTrainer:
     )
     def test_trainer_creation_from_json_config(
         self, json_file_name: str, trainer_class: type
-    ):
+    ) -> None:
         trainer = None
         file_path = pkg_resources.resource_filename(__name__, json_file_name)
         with open(file_path, "r") as parameters_file:
@@ -107,7 +104,7 @@ class TestTrainer:
         )
         assertIsInstance(trainer, trainer_class)
 
-    def test_trainer_sync_server_creation_from_json_config(self):
+    def test_trainer_sync_server_creation_from_json_config(self) -> None:
         file_path = pkg_resources.resource_filename(__name__, SYNC_TRAINER_JSON)
         with open(file_path, "r") as parameters_file:
             json_cfg = json.load(parameters_file)
@@ -130,7 +127,7 @@ class TestTrainer:
     )
     def test_trainer_creation_from_yaml_config(
         self, yaml_file_name: str, trainer_class: type
-    ):
+    ) -> None:
         trainer = None
         with initialize(config_path=CONFIG_PATH):
             cfg = compose(config_name=yaml_file_name)
@@ -141,7 +138,7 @@ class TestTrainer:
             )
         assertIsInstance(trainer, trainer_class)
 
-    def test_async_trainer_with_dp_creation_from_json_config(self):
+    def test_async_trainer_with_dp_creation_from_json_config(self) -> None:
         trainer = None
         file_path = pkg_resources.resource_filename(__name__, ASYNC_TRAINER_JSON)
         with open(file_path, "r") as parameters_file:
@@ -155,7 +152,7 @@ class TestTrainer:
         assertIsInstance(trainer, AsyncTrainer)
         assertTrue(trainer.aggregator.is_private)
 
-    def test_async_trainer_with_dp_creation_from_yaml_config(self):
+    def test_async_trainer_with_dp_creation_from_yaml_config(self) -> None:
         trainer = None
         with initialize(config_path=CONFIG_PATH):
             cfg = compose(config_name=ASYNC_TRAINER_YAML)
@@ -167,7 +164,7 @@ class TestTrainer:
         assertIsInstance(trainer, AsyncTrainer)
         assertTrue(trainer.aggregator.is_private)
 
-    def test_global_model_unchanged_after_metrics_reporting(self):
+    def test_global_model_unchanged_after_metrics_reporting(self) -> None:
         """
         reporting metrics after aggregation should NOT update the global model
         """
@@ -201,14 +198,14 @@ class TestTrainer:
         )
 
         sync_trainer_report = create_sync_trainer(
-            model=copy.deepcopy(global_model),
+            model=FLModelParamUtils.clone(global_model),
             local_lr=local_optimizer_lr,
             users_per_round=users_per_round,
             epochs=3,
             user_epochs_per_round=2,
             report_train_metrics=True,
+            report_train_metrics_after_aggregation=True,
         )
-        sync_trainer_report.cfg.report_train_metrics_after_aggregation = True
 
         data_provider = FLDataProviderFromList(
             data_loader.fl_train_set(),
@@ -223,7 +220,7 @@ class TestTrainer:
             model, _ = trainer.train(
                 data_provider,
                 metrics_reporter,
-                num_total_users=data_provider.num_users(),
+                num_total_users=data_provider.num_train_users(),
                 distributed_world_size=world_size,
             )
             modules.append(model.fl_get_module())
@@ -231,9 +228,46 @@ class TestTrainer:
         # make sure metrics reporting after aggregation does not change global model
         assertEqual(FLModelParamUtils.get_mismatched_param(modules), "")
 
-    def test_client_optimizer_creation_from_config(self):
+    def test_evaluate_global_model_after_aggregation(self):
+        (
+            data_provider,
+            _,
+        ) = DummyAlphabetDataset.create_data_provider_and_loader_train_and_eval_users(
+            num_train_examples=10,
+            num_eval_examples=20,
+            examples_per_user=1,
+            train_batch_size=1,
+            eval_batch_size=1,
+        )
+        global_model = DummyAlphabetFLModel()
+        metrics_reporter = MetricsReporterWithMockedChannels()
+        trainer = create_sync_trainer(
+            model=FLModelParamUtils.clone(global_model),
+            local_lr=1.0,
+            users_per_round=1,
+            epochs=3,
+            user_epochs_per_round=1,
+            report_train_metrics=True,
+            report_train_metrics_after_aggregation=True,
+            train_metrics_reported_per_epoch=10,
+        )
+
+        trainer.train(
+            data_provider,
+            metrics_reporter,
+            num_total_users=data_provider.num_train_users(),
+            distributed_world_size=1,
+        )
+        agg_metrics = [
+            m[0] for m in metrics_reporter.stdout_results if "Aggregation" in m[0]
+        ]
+        eval_metrics = [m[0] for m in metrics_reporter.stdout_results if "Eval" in m[0]]
+        assertEqual(len(agg_metrics), 30)
+        assertEqual(len(eval_metrics), 3)
+
+    def test_client_optimizer_creation_from_config(self) -> None:
         """
-        Test if trainer can instanciate the correct client optimizer from config
+        Test if trainer can instantiate the correct client optimizer from config
         """
         for optimizer_config, trainer_type in zip(
             [LocalOptimizerSGDConfig(lr=1.0), LocalOptimizerFedProxConfig(mu=1.0)],
@@ -261,9 +295,9 @@ class TestTrainer:
             (FedAdamAsyncAggregatorConfig(beta1=0.1), AsyncTrainer),
         ],
     )
-    def test_server_optimizer_creation_from_config(self, config, trainer_type):
+    def test_server_optimizer_creation_from_config(self, config, trainer_type) -> None:
         """
-        Test if trainer can instanciate correct aggregator config
+        Test if trainer can instantiate correct aggregator config
         """
         config = (
             SyncTrainerConfig(server=config)
@@ -273,9 +307,9 @@ class TestTrainer:
         trainer = instantiate(config, model=DummyAlphabetFLModel(), cuda_enabled=False)
         assertTrue(isinstance(trainer, trainer_type))
 
-    def test_same_training_results_with_post_aggregation_reporting(self):
+    def test_same_training_results_with_post_aggregation_reporting(self) -> None:
         """
-        creat two training instances,
+        create two training instances,
         one with report_train_metrics_after_aggregation=True, another with False,
         check training results match
         """
@@ -315,14 +349,14 @@ class TestTrainer:
             data_loader.fl_test_set(),
             global_model_1,
         )
-        assertEqual(data_provider.num_users(), data_loader.num_total_users)
+        assertEqual(data_provider.num_train_users(), data_loader.num_total_users)
         # training with reporting the train metrics after aggregation
         sync_trainer_1.cfg.report_train_metrics = True
         sync_trainer_1.cfg.report_train_metrics_after_aggregation = True
         global_model_1, best_metric_1 = sync_trainer_1.train(
             data_provider,
             metrics_reporter_1,
-            num_total_users=data_provider.num_users(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=world_size,
         )
 
@@ -343,7 +377,7 @@ class TestTrainer:
         global_model_2, best_metric_2 = sync_trainer_2.train(
             data_provider,
             metrics_reporter_2,
-            num_total_users=data_provider.num_users(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=world_size,
         )
 
@@ -355,9 +389,9 @@ class TestTrainer:
             "",
         )
 
-    def test_different_metrics_with_aggregation_client_reporting(self):
+    def test_different_metrics_with_aggregation_client_reporting(self) -> None:
         """
-        creat two training instances,
+        create two training instances,
         one with use_train_clients_for_aggregation_metrics=True, another with False,
         check that the aggregation eval metrics are different
         """
@@ -397,7 +431,7 @@ class TestTrainer:
             data_loader.fl_test_set(),
             global_model_1,
         )
-        assertEqual(data_provider.num_users(), data_loader.num_total_users)
+        assertEqual(data_provider.num_train_users(), data_loader.num_total_users)
         # training with using training clients for aggregation training metrics
         sync_trainer_1.cfg.report_train_metrics_after_aggregation = True
         sync_trainer_1.cfg.use_train_clients_for_aggregation_metrics = True
@@ -405,7 +439,7 @@ class TestTrainer:
         global_model_1, best_metric_1 = sync_trainer_1.train(
             data_provider,
             metrics_reporter_1,
-            num_total_users=data_provider.num_users(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=world_size,
         )
 
@@ -427,7 +461,7 @@ class TestTrainer:
         global_model_2, best_metric_2 = sync_trainer_2.train(
             data_provider,
             metrics_reporter_2,
-            num_total_users=data_provider.num_users(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=world_size,
         )
 
@@ -442,22 +476,22 @@ class TestTrainer:
                 return
         assert True, "Batch metrics same whether using training or random clients"
 
-    def test_one_user_sequential_user_equivalent(self):
+    def test_one_user_sequential_user_equivalent(self) -> None:
         """
         test equivalence of the following scenario,
 
         1. one user who got all the example, and BS = all examples,
         local optimizer LR = 1.0
-        2. number of users = number of examples, and one exampler per user,
+        2. number of users = number of examples, and one example per user,
         users per round = all users, use FedAvg
         """
 
         torch.manual_seed(1)
         # create dummy FL model on alphabet
         global_model = DummyAlphabetFLModel()
-        global_model_init = copy.deepcopy(global_model)
+        global_model_init = FLModelParamUtils.clone(global_model)
         # will be used later to verify training indeed took place
-        global_model_init_copy = copy.deepcopy(global_model)
+        global_model_init_copy = FLModelParamUtils.clone(global_model)
         metrics_reporter = FakeMetricReporter()
 
         num_training_examples = 32
@@ -489,7 +523,7 @@ class TestTrainer:
         one_user_global_model, _eval_metric_one_user = sync_trainer.train(
             data_provider,
             metrics_reporter,
-            num_total_users=data_provider.num_users(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=world_size,
         )
         metrics_reporter.reset()
@@ -508,9 +542,9 @@ class TestTrainer:
         assertEqual(
             data_loader.num_total_users, math.ceil(num_training_examples / shard_size)
         )
-        assertEqual(data_provider.num_users(), data_loader.num_total_users)
+        assertEqual(data_provider.num_train_users(), data_loader.num_total_users)
         # select all users to train in each round
-        users_per_round = data_provider.num_users()
+        users_per_round = data_provider.num_train_users()
         torch.manual_seed(1)
         sync_trainer = create_sync_trainer(
             model=global_model_init,
@@ -521,7 +555,7 @@ class TestTrainer:
         all_users_global_model, _eval_metric_all_user = sync_trainer.train(
             data_provider,
             metrics_reporter,
-            num_total_users=data_provider.num_users(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=world_size,
         )
         assertEqual(
@@ -535,7 +569,7 @@ class TestTrainer:
             "",
         )
 
-    def test_training_with_armijo_line_search(self):
+    def test_training_with_armijo_line_search(self) -> None:
         """
         test Armijo line-search for local LR scheduling
 
@@ -545,9 +579,9 @@ class TestTrainer:
         torch.manual_seed(1)
         # create dummy FL model on alphabet
         global_model = DummyAlphabetFLModel()
-        global_model_init = copy.deepcopy(global_model)
+        global_model_init = FLModelParamUtils.clone(global_model)
         # will be used later to verify training indeed took place
-        global_model_init_copy = copy.deepcopy(global_model)
+        global_model_init_copy = FLModelParamUtils.clone(global_model)
         metrics_reporter = FakeMetricReporter()
 
         # one user, who got 26 examples
@@ -562,7 +596,7 @@ class TestTrainer:
             dummy_dataset, shard_size, local_batch_size, global_model
         )
         assertEqual(data_loader.num_total_users, math.ceil(26 / shard_size))
-        assertEqual(data_provider.num_users(), data_loader.num_total_users)
+        assertEqual(data_provider.num_train_users(), data_loader.num_total_users)
         users_per_round = 1
         local_optimizer_lr = 1.0
         epochs = 5
@@ -577,7 +611,7 @@ class TestTrainer:
         constant_lr_model, _ = sync_trainer.train(
             data_provider,
             metrics_reporter,
-            num_total_users=data_provider.num_users(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=world_size,
         )
         metrics_reporter.reset()
@@ -599,7 +633,7 @@ class TestTrainer:
         armijo_ls_model, _ = sync_trainer_with_scheduler.train(
             data_provider,
             metrics_reporter,
-            num_total_users=data_provider.num_users(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=world_size,
         )
         metrics_reporter.reset()
@@ -621,7 +655,7 @@ class TestTrainer:
         epochs: int,
         local_lr: float,
         server_config: SyncServerConfig,
-    ):
+    ) -> None:
         """
         Given:
             data_for_fl={user1:batch1, user2:batch2}
@@ -635,7 +669,7 @@ class TestTrainer:
         # create dummy FL model on alphabet
         global_model = DummyAlphabetFLModel()
         # will be used later to verify training indeed took place
-        global_model_init_copy = copy.deepcopy(global_model)
+        global_model_init_copy = FLModelParamUtils.clone(global_model)
         # num_fl_users users, each with num_examples/num_fl_users training examples
         assertTrue(
             num_examples % num_fl_users == 0,
@@ -652,7 +686,7 @@ class TestTrainer:
             dummy_dataset, shard_size, batch_size, global_model
         )
         assertEqual(data_loader.num_total_users, math.ceil(num_examples / shard_size))
-        assertEqual(data_provider.num_users(), data_loader.num_total_users)
+        assertEqual(data_provider.num_train_users(), data_loader.num_total_users)
 
         torch.manual_seed(1)
 
@@ -666,8 +700,8 @@ class TestTrainer:
         )
         fl_model, _ = sync_trainer.train(
             data_provider,
-            metric_reporter=FakeMetricReporter(),
-            num_total_users=data_provider.num_users(),
+            metrics_reporter=FakeMetricReporter(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=world_size,
         )
 
@@ -676,7 +710,7 @@ class TestTrainer:
         data_loader = torch.utils.data.DataLoader(
             dummy_dataset, batch_size=batch_size, shuffle=False
         )
-        nonfl_model = copy.deepcopy(global_model_init_copy)
+        nonfl_model = FLModelParamUtils.clone(global_model_init_copy)
 
         optimizer = instantiate(
             config=server_config.server_optimizer,
@@ -699,7 +733,7 @@ class TestTrainer:
         )
         assertEmpty(error_msg, msg=error_msg)
 
-    def test_fl_nonfl_equivalent_global_optimizer_sgd(self):
+    def test_fl_nonfl_equivalent_global_optimizer_sgd(self) -> None:
         """
         Given:
             batch_size=13
@@ -721,7 +755,7 @@ class TestTrainer:
             ),
         )
 
-    def test_fl_nonfl_equivalent_global_optimizer_adam(self):
+    def test_fl_nonfl_equivalent_global_optimizer_adam(self) -> None:
         """
         Given:
             batch_size=16 (bs=num_examples/num_fl_users)
@@ -743,7 +777,7 @@ class TestTrainer:
             ),
         )
 
-    def test_client_overselection(self):
+    def test_client_overselection(self) -> None:
         """
         test client overselection by equivalence of the two setups:
 
@@ -760,7 +794,7 @@ class TestTrainer:
         # create dummy FL model on alphabet
         global_model = DummyAlphabetFLModel()
         # keep a copy of initial model to trigger another training instance
-        global_model_init = copy.deepcopy(global_model)
+        global_model_init = FLModelParamUtils.clone(global_model)
         # dummy alphabet dataset
         dummy_dataset = DummyAlphabetDataset()
         # two users, one gets 1 more example than the other
@@ -772,11 +806,17 @@ class TestTrainer:
         ) = DummyAlphabetDataset.create_data_provider_and_loader(
             dummy_dataset, shard_size, local_batch_size, global_model
         )
-        assertEqual(data_provider.num_users(), data_loader.num_total_users)
+        assertEqual(data_provider.num_train_users(), data_loader.num_total_users)
         # assert first user gets (dummy_dataset.num_rows / 2) + 1 data point,
         # the second user gets (dummy_dataset.num_rows / 2) - 1 data point
-        assertEqual(data_provider[0].num_examples(), dummy_dataset.num_rows / 2 + 1)
-        assertEqual(data_provider[1].num_examples(), dummy_dataset.num_rows / 2 - 1)
+        assertEqual(
+            data_provider.get_train_user(0).num_train_examples(),
+            dummy_dataset.num_rows / 2 + 1,
+        )
+        assertEqual(
+            data_provider.get_train_user(1).num_train_examples(),
+            dummy_dataset.num_rows / 2 - 1,
+        )
         # shared trainer config between two training instance
         users_per_round = 1
         local_optimizer_lr = 1.0
@@ -800,17 +840,17 @@ class TestTrainer:
         )
         model_with_overselection, _ = sync_trainer_overselection.train(
             data_provider,
-            metric_reporter=FakeMetricReporter(),
-            num_total_users=data_provider.num_users(),
+            metrics_reporter=FakeMetricReporter(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=1,
         )
         # another training instance: only user[1] in the user population
         # removing user 0 from dataset, assign user 1 to be user 0
-        data_provider._users[0] = copy.deepcopy(data_provider._users[1])
-        data_provider._users.pop(1)
+        data_provider._train_users[0] = copy.deepcopy(data_provider._train_users[1])
+        data_provider._train_users.pop(1)
         # only a single user after remove user 0
-        assertEqual(data_provider.num_users(), 1)
-        global_model = copy.deepcopy(global_model_init)
+        assertEqual(data_provider.num_train_users(), 1)
+        global_model = FLModelParamUtils.clone(global_model_init)
         torch.manual_seed(1)
         dropout_rate = 1.0
         epochs = 2
@@ -823,8 +863,8 @@ class TestTrainer:
         )
         model_single_user, _ = sync_trainer_single_user.train(
             data_provider,
-            metric_reporter=FakeMetricReporter(),
-            num_total_users=data_provider.num_users(),
+            metrics_reporter=FakeMetricReporter(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=1,
         )
         assertEqual(
@@ -838,21 +878,21 @@ class TestTrainer:
             "",
         )
 
-    def test_partial_update_from_clients(self):
+    def test_partial_update_from_clients(self) -> None:
         """
         test the equivalence of these two training instance
         1. UPR=1. User dataset has 52 characters [a,b,c,d .... x, y ,z, a, b, c... x,y,z],
         where each character appears twice in order. Timeout limit is set to
         just enough for training half of the dataset
 
-        2. UPR=1. User daraset has 26 characters [a,b,c,d ...,x,y,z]. no timeout
+        2. UPR=1. User dataset has 26 characters [a,b,c,d ...,x,y,z]. no timeout
         """
         torch.manual_seed(1)
         # create dummy FL model on alphabet
         global_model = DummyAlphabetFLModel()
         # keep a copy of initial model to trigger another training instance
-        global_model_init = copy.deepcopy(global_model)
-        # dummy alphabet dataset, getting each character twrice
+        global_model_init = FLModelParamUtils.clone(global_model)
+        # dummy alphabet dataset, getting each character twice
         num_rows = 52
         dummy_dataset = DummyAlphabetDataset(num_rows)
         # a single user getting all examples
@@ -869,10 +909,10 @@ class TestTrainer:
         local_optimizer_lr = 1.0
         epochs = 1
         torch.manual_seed(1)
-        # training time just enought for 26 examples, although user has
+        # training time just enough for 26 examples, although user has
         # 52 examples
         sync_trainer_timeout = create_sync_trainer(
-            model=copy.deepcopy(global_model),
+            model=FLModelParamUtils.clone(global_model),
             local_lr=local_optimizer_lr,
             users_per_round=users_per_round,
             epochs=epochs,
@@ -886,8 +926,8 @@ class TestTrainer:
         )
         model_with_timeout, _ = sync_trainer_timeout.train(
             data_provider,
-            metric_reporter=FakeMetricReporter(),
-            num_total_users=data_provider.num_users(),
+            metrics_reporter=FakeMetricReporter(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=1,
         )
         # dummy alphabet dataset, getting each character once
@@ -904,17 +944,17 @@ class TestTrainer:
             dummy_dataset, shard_size, local_batch_size, global_model
         )
         torch.manual_seed(1)
-        # training time just enought for 26 examples
+        # training time just enough for 26 examples
         sync_trainer_timeout = create_sync_trainer(
-            model=copy.deepcopy(global_model),
+            model=FLModelParamUtils.clone(global_model),
             local_lr=local_optimizer_lr,
             users_per_round=users_per_round,
             epochs=epochs,
         )
         model_no_timeout, _ = sync_trainer_timeout.train(
             data_provider,
-            metric_reporter=FakeMetricReporter(),
-            num_total_users=data_provider.num_users(),
+            metrics_reporter=FakeMetricReporter(),
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=1,
         )
         assertEqual(
@@ -928,7 +968,7 @@ class TestTrainer:
             "",
         )
 
-    def test_client_metric_reporting(self):
+    def test_client_metric_reporting(self) -> None:
         """
         Test that per-client reporting reports exactly every
         ``client_metrics_reported_per_epoch`` as defined in the config.
@@ -936,7 +976,7 @@ class TestTrainer:
         torch.manual_seed(1)
         # create dummy FL model on alphabet
         global_model = DummyAlphabetFLModel()
-        # dummy alphabet dataset, getting each character twrice
+        # dummy alphabet dataset, getting each character twice
         num_rows = 52
         num_users = 4
         dummy_dataset = DummyAlphabetDataset(num_rows)
@@ -990,8 +1030,8 @@ class TestTrainer:
 
         model, _ = sync_trainer_with_client_reports.train(
             data_provider,
-            metric_reporter=metrics_reporter,
-            num_total_users=data_provider.num_users(),
+            metrics_reporter=metrics_reporter,
+            num_total_users=data_provider.num_train_users(),
             distributed_world_size=1,
         )
 
@@ -1006,8 +1046,82 @@ class TestTrainer:
             metrics_reporter.stdout_results,
         )
 
+    def test_average_overflow_metric_reporting(self) -> None:
+        """
+        Tests that average overflow metric reporting occurs correctly.
+        """
+        torch.manual_seed(1)
+        # create dummy FL model on alphabet
+        global_model = DummyAlphabetFLModel()
+        # dummy alphabet dataset
+        num_rows = 30
+        num_users = 16
+        dummy_dataset = DummyAlphabetDataset(num_rows)
+        shard_size = int(len(dummy_dataset) / num_users)
+        local_batch_size = 1
+        (
+            data_provider,
+            data_loader,
+        ) = DummyAlphabetDataset.create_data_provider_and_loader(
+            dummy_dataset, shard_size, local_batch_size, global_model
+        )
+        users_per_round = 2  # we will have 8 rounds per epoch
+        epochs = 6
+        train_metrics_reported_per_epoch = 4
+
+        sync_trainer = SyncTrainer(
+            model=global_model,
+            cuda_enabled=False,
+            **OmegaConf.structured(
+                SyncTrainerConfig(
+                    epochs=epochs,
+                    do_eval=True,
+                    users_per_round=users_per_round,
+                    always_keep_trained_model=False,
+                    train_metrics_reported_per_epoch=train_metrics_reported_per_epoch,
+                    eval_epoch_frequency=1,
+                    report_train_metrics=True,
+                    report_train_metrics_after_aggregation=True,
+                    client=ClientConfig(
+                        epochs=1,
+                        optimizer=LocalOptimizerSGDConfig(lr=1.0, momentum=0.0),
+                        max_clip_norm_normalized=None,
+                    ),
+                    server=SyncSecAggServerConfig(
+                        server_optimizer=FedAvgWithLROptimizerConfig(
+                            lr=1.0, momentum=0.0
+                        ),
+                        active_user_selector=SequentialActiveUserSelectorConfig(),
+                        fixedpoint=FixedPointConfig(num_bytes=2, scaling_factor=1000),
+                    ),
+                    report_client_metrics=False,
+                    report_client_metrics_after_epoch=False,
+                )
+            ),
+        )
+        metrics_reporter = MetricsReporterWithMockedChannels()
+
+        assertEqual(sync_trainer._last_report_round_after_aggregation, 0)
+        model, _ = sync_trainer.train(
+            data_provider,
+            metrics_reporter=metrics_reporter,
+            num_total_users=data_provider.num_train_users(),
+            distributed_world_size=1,
+        )
+
+        def count_word(result, word):
+            return str(result).count(word)
+
+        # If train_metrics_reported_per_epoch = 4 and number of epochs = 6
+        # train eval metrics (including overflow) should be reported 4*6 = 24 times
+        assertEqual(
+            count_word(metrics_reporter.stdout_results, "overflow per round"),
+            24,
+            metrics_reporter.stdout_results,
+        )
+
     def _get_tensorboard_results_from_training(
-        self, num_total_users, num_epochs, users_per_round
+        self, num_total_users: int, num_epochs: int, users_per_round: int
     ) -> List[MockRecord]:
         # dataset has 26 rows
         assertTrue(num_total_users <= 26, "Can't have more than 26 users")
@@ -1016,20 +1130,20 @@ class TestTrainer:
         shard_size = int(math.ceil(26 / num_total_users))
         local_batch_size = 4
         dummy_dataset = DummyAlphabetDataset()
-        fl_data_sharder = SequentialSharder(examples_per_shard=shard_size)
-        data_loader = FLDatasetDataLoaderWithBatch(
+        global_model = DummyAlphabetFLModel()
+
+        (
+            data_provider,
+            data_loader,
+        ) = DummyAlphabetDataset.create_data_provider_and_loader(
             dummy_dataset,
-            dummy_dataset,
-            dummy_dataset,
-            fl_data_sharder,
-            local_batch_size,
-            local_batch_size,
-            local_batch_size,
+            examples_per_user=shard_size,
+            batch_size=local_batch_size,
+            model=global_model,
         )
 
+        assertEqual(data_provider.num_train_users(), data_loader.num_total_users)
         torch.manual_seed(1)
-        # first training instance
-        global_model = DummyAlphabetFLModel()
         metrics_reporter = MetricsReporterWithMockedChannels()
         sync_trainer = create_sync_trainer(
             model=global_model,
@@ -1037,16 +1151,10 @@ class TestTrainer:
             users_per_round=users_per_round,
             epochs=num_epochs,
             user_epochs_per_round=1,
+            report_train_metrics=True,
+            report_train_metrics_after_aggregation=True,
         )
-        data_provider = FLDataProviderFromList(
-            data_loader.fl_train_set(),
-            data_loader.fl_eval_set(),
-            data_loader.fl_test_set(),
-            global_model,
-        )
-        assertEqual(data_provider.num_users(), data_loader.num_total_users)
-        sync_trainer.cfg.report_train_metrics = True
-        sync_trainer.cfg.report_train_metrics_after_aggregation = True
+
         global_model, best_metric = sync_trainer.train(
             data_provider,
             metrics_reporter,
@@ -1055,7 +1163,7 @@ class TestTrainer:
         )
         return metrics_reporter.tensorboard_results
 
-    def test_tensorboard_metrics_reporting_simple(self):
+    def test_tensorboard_metrics_reporting_simple(self) -> None:
         """Train with tensorboard metrics reporter for one epoch, 5 rounds per epoch.
         Test for 2 things:
         a) Train, Aggregation and Eval metrics are reported once
@@ -1095,7 +1203,7 @@ class TestTrainer:
             f"Actual global steps: {global_steps_reported_actual}, Expected global steps:{global_steps_reported_expected}",
         )
 
-    def test_tensorboard_metrics_reporting_complex(self):
+    def test_tensorboard_metrics_reporting_complex(self) -> None:
         """Train with tensorboard metrics reporter. Ensure eval and train metrics are
         correctly reported to tensorboard
         """
@@ -1147,63 +1255,3 @@ class TestTrainer:
             global_steps_reported_expected,
             f"Actual global steps: {global_steps_reported_actual}, Expected global steps:{global_steps_reported_expected}",
         )
-
-    @pytest.mark.parametrize(
-        "page_turn_freq,users_per_round, pages_used",
-        [
-            (0.99, 5, 6),
-            (0.99, 10, 6),
-            (0.50, 10, 9),
-            (0.50, 5, 9),
-        ],
-    )
-    def test_sync_trainer_with_page_data_provider(
-        self, page_turn_freq, users_per_round, pages_used
-    ):
-        """
-        Test if sync trainer works properly with paged data loader
-        Assumptions:
-            users_per_page = 10 total number of users = 50
-            batch_size = 1
-            train for 2 global epochs
-
-        Cases:
-        1. Base case: user_per_round is half of page_size, turn page after all users are used
-        2. Boundary case: users_per_round = page_size, turn page after all users are used
-        3. Boundary case: users_per_round = page_size, turn page after half of users in the page are used
-        """
-        id_col = "label"
-        num_total_users = 50
-        inline_config = InlineDatasetConfig(
-            num_users=num_total_users,
-            data_cols=["user_n"],
-            id_col=id_col,
-        )
-        dataloader = create_dataloader(
-            num_users_per_page=10,
-            num_total_users=num_total_users,
-            batch_size=1,
-            inline_config=inline_config,
-            sharding_col_name=id_col,
-            use_nid=True,
-        )
-        global_model = SampleNetHive()
-        data_provider = PagedDataProvider(
-            dataloader, model=global_model, page_turn_freq=page_turn_freq
-        )
-        sync_trainer = create_sync_trainer(
-            model=global_model,
-            local_lr=0.1,
-            users_per_round=users_per_round,
-            epochs=1,
-        )
-
-        sync_trainer.train(
-            data_provider,
-            metric_reporter=FakeMetricReporter(),
-            # Note: We're using num_total_users and
-            # not data_provider.num_users()
-            num_total_users=num_total_users,
-            distributed_world_size=1,
-        )
-        assertEqual(data_provider.pages_used, pages_used)
