@@ -1,11 +1,12 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
 from dataclasses import dataclass
-from enum import Enum, auto
+from enum import auto, Enum
 
 from flsim.utils.async_trainer.training_event_generator import IEventGenerator
 
