@@ -1,20 +1,27 @@
 #!/usr/bin/env python3
 # Copyright (c) Meta Platforms, Inc. and affiliates.
+# All rights reserved.
 #
 # This source code is licensed under the BSD-style license found in the
 # LICENSE file in the root directory of this source tree.
 
 import copy
+import logging
 import math
-from typing import List, Optional
+from typing import List, Optional, Union
 
 import torch
+from flsim.common.logger import Logger
+from flsim.interfaces.model import IFLModel
 from flsim.utils.fl.personalized_model import FLModelWithPrivateModules
 from torch import nn
 from torch.optim.optimizer import Optimizer
 
 
 class FLModelParamUtils:
+    logger: logging.Logger = Logger.get_logger(__name__)
+    logger.setLevel(logging.WARNING)
+
     @classmethod
     def get_state_dict(cls, model: nn.Module, only_federated_params: bool):
         if only_federated_params and isinstance(model, FLModelWithPrivateModules):
@@ -50,7 +57,7 @@ class FLModelParamUtils:
         abs_epsilon: Optional[float] = None,
     ) -> str:
         """Compare all the models in the given list of models.
-        It returns an empty string if all the models have same parameters.
+        It returns an empty string if all the models have the same parameters.
         It returns the name of the first parameter that is different if any.
         """
         if rel_epsilon is None and abs_epsilon is not None:
@@ -59,7 +66,7 @@ class FLModelParamUtils:
         if len(models) <= 1:
             return ""
         dicts = [aModel.state_dict() for aModel in models]
-        # verify new models have all params same
+        # Verify new models have all params same
         rtol_atol = {}
         if rel_epsilon is not None:
             rtol_atol["rtol"] = rel_epsilon
@@ -67,11 +74,11 @@ class FLModelParamUtils:
             rtol_atol["atol"] = abs_epsilon
         for name, param in dicts[0].items():
             for adict in dicts[1:]:
-                # if a parameter name does not exist in a model, return early
+                # If a parameter name does not exist in a model, return early
                 if name not in adict.keys():
                     return name
                 param_here = adict[name]
-                # if epsilon is specified, do approx comparison
+                # If epsilon is specified, do approx comparison
                 if not torch.allclose(param.float(), param_here.float(), **rtol_atol):
                     return name
         return ""
@@ -116,11 +123,9 @@ class FLModelParamUtils:
             models: collection of models. These will be changed in-place
             model_to_save: update this model with the average
             weights: (optional) use weighted average
-        Returns:
-            none
         """
         assert weights is None or len(weights) == len(models), (
-            "Weights should have same length as models. len(wts):"
+            "Weights should have the same length as models. len(wts):"
             + str(len(weights))
             + ", len(models):"
             + str(len(models))
@@ -152,29 +157,31 @@ class FLModelParamUtils:
             from_model: a model
             to_models: collection of models. These will be changed in-place
             only_federated_params: copy only federated params.
-        Returns:
-            none
         """
         from_state_dict = cls.get_state_dict(from_model, only_federated_params)
         for m in to_models:
             cls.load_state_dict(m, from_state_dict, only_federated_params)
 
     @classmethod
-    def clone(cls, model: nn.Module, dtype: Optional[torch.dtype] = None):
-        """
-        Clones a pytorch module, and allows for a change of precision.
+    def clone(
+        cls, model: Union[nn.Module, IFLModel], dtype: Optional[torch.dtype] = None
+    ):
+        """Clones a pytorch module, and allows for a change of precision.
         TODO If needed we can also add device here.
         """
         new_model = copy.deepcopy(model)
-        return (
-            new_model.float()  # pyre-ignore
-            if dtype == torch.float32
-            else (
-                new_model.double()  # pyre-ignore
-                if dtype == torch.float64
-                else new_model
+        if isinstance(new_model, IFLModel):
+            if dtype == torch.float32:
+                new_model.fl_get_module().float()
+            elif dtype == torch.float64:
+                new_model.fl_get_module().double()
+            return new_model
+        else:
+            return (
+                new_model.float()
+                if dtype == torch.float32
+                else (new_model.double() if dtype == torch.float64 else new_model)
             )
-        )
 
     @classmethod
     def set_gradient(cls, model: nn.Module, reference_gradient: nn.Module) -> None:
@@ -183,10 +190,122 @@ class FLModelParamUtils:
             model: nn.Module
             reference_gradient: nn.Module - gradient is the parameters of this model
         """
-        # Use parameters() since state_dict() may inlude non-learnable params.
+        # Use parameters() since state_dict() may include non-learnable params.
         for m, ref in zip(model.parameters(), reference_gradient.parameters()):
             m.grad = ref.detach().clone().type(m.type())
 
+    @classmethod
+    def linear_combine_gradient(
+        cls,
+        model1: nn.Module,
+        wt1: float,
+        model2: nn.Module,
+        wt2: float,
+        model_to_save: nn.Module,
+    ):
+        """Sets model_to_save.grad = model1.grad * wt1 + model2.grad * wt2"""
+        for save_p, model1_p, model2_p in zip(
+            model_to_save.parameters(), model1.parameters(), model2.parameters()
+        ):
+            if save_p.requires_grad:
+                grad = None
+                if model1_p.grad is not None:
+                    grad = wt1 * model1_p.grad
+                if model2_p.grad is not None:
+                    if grad is not None:
+                        grad += wt2 * model2_p.grad
+                    else:
+                        grad = wt2 * model2_p.grad
+                if grad is None:
+                    cls.logger.warning(
+                        "Parameter with requires_grad=True has gradient set to None"
+                    )
+                save_p.grad = grad
+
+    @classmethod
+    def multiply_gradient_by_weight(
+        cls, model: nn.Module, weight: float, model_to_save: nn.Module
+    ):
+        """Sets model_to_save.grad = model.grad * weight"""
+        for save_p, model_p in zip(model_to_save.parameters(), model.parameters()):
+            if save_p.requires_grad:
+                grad = None
+                if model_p.grad is not None:
+                    grad = weight * model_p.grad
+                if grad is None:
+                    cls.logger.warning(
+                        "Parameter with requires_grad=True has gradient set to None"
+                    )
+                del save_p.grad
+                save_p.grad = grad
+
+    @classmethod
+    def add_gradients(
+        cls, model1: nn.Module, model2: nn.Module, model_to_save: nn.Module
+    ):
+        """Sets model_to_save.grad = model1.grad + model2.grad"""
+        for save_p, model1_p, model2_p in zip(
+            model_to_save.parameters(), model1.parameters(), model2.parameters()
+        ):
+            if save_p.requires_grad:
+                grad = None
+                if model1_p.grad is not None:
+                    grad = model1_p.grad.detach().clone().type(save_p.type())
+                if model2_p.grad is not None:
+                    if grad is not None:
+                        grad += model2_p.grad
+                    else:
+                        grad = model2_p.grad.detach().clone().type(save_p.type())
+                if grad is None:
+                    cls.logger.warning(
+                        "Parameter with requires_grad=True has gradient set to None"
+                    )
+                del save_p.grad
+                save_p.grad = grad
+
+    @classmethod
+    def subtract_gradients(
+        cls, minuend: nn.Module, subtrahend: nn.Module, difference: nn.Module
+    ):
+        """Sets difference.grad = minuend.grad - subtrahend.grad"""
+        for difference_p, minuend_p, subtrahend_p in zip(
+            difference.parameters(), minuend.parameters(), subtrahend.parameters()
+        ):
+            if difference_p.requires_grad:
+                grad = None
+                if minuend_p.grad is not None:
+                    grad = minuend_p.grad
+                if subtrahend_p.grad is not None:
+                    if grad is not None:
+                        grad -= subtrahend_p.grad
+                    else:
+                        grad = (
+                            -subtrahend_p.grad.detach()
+                            .clone()
+                            .type(difference_p.type())
+                        )
+                if grad is None:
+                    cls.logger.warning(
+                        "Parameter with requires_grad=True has gradient set to None"
+                    )
+                del difference_p.grad
+                difference_p.grad = grad
+
+    @classmethod
+    def copy_gradients(cls, model: nn.Module, model_to_copy: nn.Module):
+        """Sets model_to_copy.grad = model.grad"""
+        for copy_p, model_p in zip(model_to_copy.parameters(), model.parameters()):
+            if copy_p.requires_grad:
+                grad = None
+                if model_p.grad is not None:
+                    grad = model_p.grad.detach().clone().type(copy_p.type())
+                if grad is None:
+                    cls.logger.warning(
+                        "Parameter with requires_grad=True has gradient set to None"
+                    )
+                del copy_p.grad
+                copy_p.grad = grad
+
     @classmethod
     def reconstruct_gradient(
         cls, old_model: nn.Module, new_model: nn.Module, grads: nn.Module
@@ -199,6 +318,10 @@ class FLModelParamUtils:
     def get_trainable_params(cls, model: nn.Module):
         return filter(lambda p: p.requires_grad, model.parameters())
 
+    @classmethod
+    def get_trainable_named_parameters(cls, model: nn.Module):
+        return filter(lambda np: np[1].requires_grad, model.named_parameters())
+
     @classmethod
     def get_gradient_l2_norm_raw(cls, model: nn.Module) -> float:
         total_norm = 0
@@ -268,8 +391,6 @@ class FLModelParamUtils:
     ) -> None:
         for param_group in optimizer.param_groups:
             param_group["lr"] = base_lr * lr_normalizer
-        # pyre-ignore[20] Call `Optimizer.step` expects argument `closure`
-        # but closure is optional in torch.optim.Optimizer
         optimizer.step()
 
     @classmethod
@@ -280,10 +401,8 @@ class FLModelParamUtils:
         model_to_save: nn.Module,
         only_federated_params: bool = False,
     ):
-        """
-        Returns model_to_save = model * weight
-        """
-        FLModelParamUtils.linear_comb_models(
+        """Returns model_to_save = model * weight."""
+        cls.linear_comb_models(
             model, weight, model, 0, model_to_save, only_federated_params
         )
 
@@ -295,10 +414,8 @@ class FLModelParamUtils:
         difference: nn.Module,
         only_federated_params: bool = False,
     ):
-        """
-        Returns difference = minuend - subtrahend
-        """
-        FLModelParamUtils.linear_comb_models(
+        """Returns difference = minuend - subtrahend."""
+        cls.linear_comb_models(
             minuend, 1, subtrahend, -1, difference, only_federated_params
         )
 
@@ -313,6 +430,6 @@ class FLModelParamUtils:
         """
         Returns model_to_save = model1 + model2
         """
-        FLModelParamUtils.linear_comb_models(
+        cls.linear_comb_models(
             model1, 1, model2, 1, model_to_save, only_federated_params
         )
